@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: add.c,v 1.21 2005/03/03 20:48:44 tobiasoed Exp $
+ * $Id: add.c,v 1.22 2006/05/29 13:18:52 tobiasoed Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -398,13 +398,17 @@ AddCB(Widget w, AddData addData, XtPointer call_data)
     case RowOrientation:
 	XbaeMatrixUnhighlightRow(addData->matrix, addData->row);
 
-	rows = (String *) XtMalloc(addData->num_columns * addData->number *
-				   sizeof(String));
-	for (i = 0; i < addData->number; i++)
-	    for (j = 0; j < addData->num_columns; j++) {
-		sprintf(buf, "addrow %d, %d", i, j);
-		rows[i * addData->num_columns + j] = XtNewString(buf);
-	    }
+        rows = NULL;
+        if (addData->num_columns > 0) {
+	    rows = (String *) XtMalloc(addData->num_columns * addData->number *
+				       sizeof(String));
+	    for (i = 0; i < addData->number; i++) {
+	        for (j = 0; j < addData->num_columns; j++) {
+		    sprintf(buf, "addrow %d, %d", i, j);
+		    rows[i * addData->num_columns + j] = XtNewString(buf);
+	        }
+            }
+        }
 
 	XbaeMatrixAddVarRows(addData->matrix, addData->row,
 			     rows, NULL, sizes, NULL, NULL, NULL, NULL,
@@ -422,13 +426,17 @@ AddCB(Widget w, AddData addData, XtPointer call_data)
     case ColumnOrientation:
 	XbaeMatrixUnhighlightColumn(addData->matrix, addData->column);
 
-	columns = (String *) XtMalloc(addData->num_rows * addData->number *
-				      sizeof(String));
-	for (i = 0; i < addData->num_rows; i++)
-	    for (j = 0; j < addData->number; j++) {
-		sprintf(buf, "addcol %d, %d", i, j);
-		columns[i * addData->number + j] = XtNewString(buf);
-	    }
+        columns = NULL;
+        if (addData->num_rows > 0) {
+	    columns = (String *) XtMalloc(addData->num_rows * addData->number *
+				          sizeof(String));
+	    for (i = 0; i < addData->num_rows; i++) {
+	        for (j = 0; j < addData->number; j++) {
+		    sprintf(buf, "addcol %d, %d", i, j);
+		    columns[i * addData->number + j] = XtNewString(buf);
+	        }
+            }
+        }
 
 	XbaeMatrixAddColumns(addData->matrix, addData->column,
 			     columns, NULL, sizes, NULL, NULL, NULL, NULL,
