@@ -1,7 +1,7 @@
 /*
  * AUTHOR: Jay Schmidgall <jay.schmidgall@spdbump.sungardss.com>
  *
- * $Id: choice.c,v 1.33 2005/08/02 15:00:53 tobiasoed Exp $
+ * $Id: choice.c,v 1.38 2006/05/16 10:52:46 tobiasoed Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,7 @@
 #include <dmalloc.h>
 #endif
 
-#define USE_RENDER_TABLE 1
+#define USE_RENDER_TABLE 0
 #define VERBOSE 0
 
 #if (XmVERSION < 2)
@@ -67,7 +67,7 @@ static String fallback[] = {
     "Choice*mw.multiLineCell: True",
     "Choice*mw.wrapType: wrap_none",
     "Choice*mw.showArrows: True",
-    #if 1
+    #if 0
     #if USE_RENDER_TABLE
     "Choice*mw.renderTable: labels",
     "Choice*mw.renderTable.fontType:         FONT_IS_FONT",
@@ -75,8 +75,8 @@ static String fallback[] = {
     "Choice*mw.renderTable.labels.fontType:  FONT_IS_FONT",
     "Choice*mw.renderTable.labels.fontName: -*-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*",
     #else
-    "Choice*mw.labelFont:		-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*",
-	"Choice*mw.fontList:		-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*",
+	"Choice*mw.fontList:		-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*",
+    "Choice*mw.labelFont:		-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*",
     #endif
     #else
     "Choice*form.textRenderTable: labels",
@@ -88,6 +88,16 @@ static String fallback[] = {
     #if 0
     "Choice*mw.sensitive: False",
     #endif
+    "Choice*mw.translations:	"
+    "                            <Btn1Up>           :   DefaultAction()\\n"
+    "                            <Btn1Down>         :   DefaultAction() Label() EditCell(Pointer) SelectCell(cell)\\n"
+    "                            Shift<Btn2Down>    :   ResizeColumns()\\n"
+    "                            <Btn4Down>         :   ScrollRows(-50)\\n"
+    "                            <Btn5Down>         :   ScrollRows( 50)\\n",
+	#if 0
+	"Choice*mw.textTranslations:	"
+    "                            <Key>Return    :   CommitEdit(True)\\n",
+    #endif
     NULL
 };
 
@@ -152,11 +162,6 @@ struct name_int {
     int value;
 };
 
-struct name_string {
-    char *name;
-    char *value;
-};
-
 enum { SelectSelect, SelectHighlight, SelectArm };
 
 /* ================================================================================================
@@ -472,11 +477,12 @@ cbScale(Widget w, XtPointer client, XtPointer call)
 }
 
 Widget
-createScaleBox(Widget mw, Widget parent, char *name, char *resource, int min, int max)
+createScaleBox(Widget mw, Widget parent, char *name, char *resource, int min, int max, Boolean is_int)
 {
     Widget frame, frame2, label, scale;
     SetValueStruct *svs;
     Dimension value;
+    int value_int;
 
     frame = XtVaCreateManagedWidget(
 	"frame", xmFrameWidgetClass, parent,
@@ -494,7 +500,12 @@ createScaleBox(Widget mw, Widget parent, char *name, char *resource, int min, in
 	name, xmLabelWidgetClass, frame2,
 	NULL);
 
-    XtVaGetValues(mw, resource, &value, NULL);
+    if (is_int) {
+        XtVaGetValues(mw, resource, &value_int, NULL);
+    } else {
+        XtVaGetValues(mw, resource, &value, NULL);
+        value_int = value;
+    }
 
     scale = XtVaCreateManagedWidget(
 	"scale", xmScaleWidgetClass, frame,
@@ -504,11 +515,15 @@ createScaleBox(Widget mw, Widget parent, char *name, char *resource, int min, in
 	XmNminimum,	    min,
 	XmNmaximum,	    max,
 	XmNscaleMultiple, 1,
-    XmNvalue, value,
+    XmNvalue, value_int,
 	NULL);
 
     #if VERBOSE
-    fprintf(stderr,"%s (dimension) %hd\n", resource, value);
+    if (is_int) {
+        fprintf(stderr,"%s (int) %d\n", resource, value_int);
+    } else {
+        fprintf(stderr,"%s (dimension) %hd\n", resource, value);
+    }
     #endif
 
     svs = XtNew(SetValueStruct);
@@ -893,6 +908,15 @@ sortCB(Widget w, XtPointer client, XtPointer call)
         }
 }
 
+void cbDefault(Widget w, XtPointer client, XtPointer call) {
+    Widget text = (Widget) client;
+    XbaeMatrixDefaultActionCallbackStruct *cbs =
+        (XbaeMatrixDefaultActionCallbackStruct*) call;
+    char buf[200];
+    sprintf(buf, "Double click in cell %d,%d", cbs->row, cbs->column);
+    XmTextSetString(text, buf);
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -952,11 +976,35 @@ main(int argc, char *argv[])
 	XmNrightAttachment,	XmATTACH_OPPOSITE_FORM,
 	XmNrightOffset,	4,
 	NULL);
-    XtOverrideTranslations(mw,
-			   XtParseTranslationTable(
-			":<Btn1Down>: SelectCell(cell) EditCell(Pointer)"));
 
+    #if 1
+        #if 0
+        {
+            short widths[] =  {0, 140, 140, 0, 70, 70, 70, 70, 70, 70, 70};
+            short heights[] = {0,  10,  10, 0, 10, 10, 10, 10, 10, 10, 10};
+            XtVaSetValues(mw, 
+                          XmNcolumnWidthInPixels, True, 
+                          XmNrowHeightInPixels, True, 
+                          XmNcolumnWidths, widths, 
+                          XmNrowHeights, heights,
+                          NULL);
+        }
+        #else
+        {
+            short widths[] =  {0, 14, 14, 0, 7, 7, 7, 7, 7, 7, 7};
+            short heights[] = {0,  1,  1, 0, 1, 1, 1, 1, 1, 1, 1};
+            XtVaSetValues(mw, 
+                          XmNcolumnWidthInPixels, False, 
+                          XmNrowHeightInPixels, False, 
+                          XmNcolumnWidths, widths, 
+                          XmNrowHeights, heights,
+                          NULL);
+        }
+        #endif
+    #endif
+    
     XtAddCallback(mw, XmNselectCellCallback, cbSelect, (XtPointer) text);
+    XtAddCallback(mw, XmNdefaultActionCallback, cbDefault, (XtPointer) text);
     
     combos = XtVaCreateManagedWidget(
 	"rc", xmRowColumnWidgetClass, form,
@@ -989,19 +1037,20 @@ main(int argc, char *argv[])
 	XmNnumColumns,		2,
 	XmNpacking,		XmPACK_COLUMN,
 	NULL);
+
+    createScaleBox(mw, sliders, "Space", XmNspace, 0, 10, False);
+    createScaleBox(mw, sliders, "Text Shadow Thickness", XmNtextShadowThickness, 0, 5, False);
+    createScaleBox(mw, sliders, "Cell Shadow Thickness", XmNcellShadowThickness, 0, 10, False);
+    createScaleBox(mw, sliders, "Cell Margin Width", XmNcellMarginWidth, 0, 10, False);
+    createScaleBox(mw, sliders, "Fixed Rows", XmNfixedRows, 0, 5, False);
+    createScaleBox(mw, sliders, "Trailing Fixed Rows", XmNtrailingFixedRows, 0, 5, False);
     
-    createScaleBox(mw, sliders, "Shadow Thickness", XmNshadowThickness,0, 10);
-    createScaleBox(mw, sliders, "Cell Shadow Thickness", XmNcellShadowThickness, 0, 10);
-    createScaleBox(mw, sliders, "Cell Margin Height", XmNcellMarginHeight, 0, 10);
-    createScaleBox(mw, sliders, "Fixed Rows", XmNfixedRows, 0, 5);
-    createScaleBox(mw, sliders, "Trailing Fixed Rows", XmNtrailingFixedRows, 0, 5);
-
-    createScaleBox(mw, sliders, "Space", XmNspace, 0, 10);
-    createScaleBox(mw, sliders, "Cell Highlight Thickness", XmNcellHighlightThickness, 0, 4);
-    createScaleBox(mw, sliders, "Cell Margin Width", XmNcellMarginWidth, 0, 10);
-    createScaleBox(mw, sliders, "Fixed Columns", XmNfixedColumns, 0, 5);
-    createScaleBox(mw, sliders, "Trailing Fixed Columns", XmNtrailingFixedColumns, 0, 5);
-
+    createScaleBox(mw, sliders, "Atl Row Count", XmNaltRowCount, 0, 5, True);
+    createScaleBox(mw, sliders, "Shadow Thickness", XmNshadowThickness,0, 10, False);
+    createScaleBox(mw, sliders, "Cell Highlight Thickness", XmNcellHighlightThickness, 0, 4, False);
+    createScaleBox(mw, sliders, "Cell Margin Height", XmNcellMarginHeight, 0, 10, False);
+    createScaleBox(mw, sliders, "Fixed Columns", XmNfixedColumns, 0, 5, False);
+    createScaleBox(mw, sliders, "Trailing Fixed Columns", XmNtrailingFixedColumns, 0, 5, False);
 
     toggles = XtVaCreateManagedWidget(
 	"rc", xmRowColumnWidgetClass, form,
@@ -1014,7 +1063,7 @@ main(int argc, char *argv[])
 	XmNnumColumns,		2,
 	XmNpacking,		XmPACK_COLUMN,
 	NULL);
-    
+
     createToggle(mw, toggles, "Fill", XmNfill);
     createToggle(mw, toggles, "Horzizontal Fill", XmNhorzFill);
     createToggle(mw, toggles, "Trailing Attached Right", XmNtrailingAttachedRight);
@@ -1024,6 +1073,7 @@ main(int argc, char *argv[])
 
     createToggle(mw, toggles, "Reverse Select", XmNreverseSelect);
     createToggle(mw, toggles, "Text background is cell", XmNtextBackgroundIsCell);
+    createToggle(mw, toggles, "Traverse Fixed Cells", XmNtraverseFixedCells);
 
     createToggle(mw, toggles, "Button Labels", XmNbuttonLabels);
     createToggle(mw, toggles, "Vertical Fill", XmNvertFill);
