@@ -1,7 +1,7 @@
 /*
  * AUTHOR: Jay Schmidgall <jay.schmidgall@spdbump.sungardss.com>
  *
- * $Id: fifteen.c,v 1.9 2005/05/23 16:09:24 tobiasoed Exp $
+ * $Id: fifteen.c,v 1.10 2006/05/15 15:57:04 tobiasoed Exp $
  */
 #ifdef HAVE_CONFIG_H
 #include <XbaeConfig.h>
@@ -21,7 +21,6 @@
 
 #include <Xbae/Matrix.h>
 
-
 static String fallback[] = {
 	"Fifteen*mw.gridType:			grid_cell_shadow",
 	"Fifteen*mw.cellShadowType:	shadow_out",
@@ -36,8 +35,6 @@ static String fallback[] = {
 	"Fifteen*mw.traversalOn:		off",
 	"Fifteen*mw.translations:		#replace\\n"
 	"			<Btn1Down>:	SelectCell()",
-	"Fifteen*mw.clip.translations:	#replace\\n"
-	"			<Btn1Down>:	SelectCell()",
 	NULL
 };
 
@@ -54,7 +51,7 @@ LoadMatrix(Widget w)
 	{ "3",	"1",	"6",	"2"},
 	{ "5",  "7",	"15",   "13"},
 	{ "4",  "11",	"8",    "9"},
-	{ "14", "10",	"MM",   ""}
+	{ "14", "10",	"12",   ""}
     };
 
     cells[0] = &rows[0][0];
@@ -183,7 +180,7 @@ main(int argc, char *argv[])
     
     XtVaGetValues(mw, XmNrows, &rows, XmNcolumns, &columns, NULL);
 
-    empty->row = rows-1;
+    empty->row = rows - 1;
     empty->column = columns - 1;
     empty->count = 0;
     
