@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: select-drag.c,v 1.10 2005/03/10 20:51:48 tobiasoed Exp $
+ * $Id: select-drag.c,v 1.12 2006/05/16 10:52:46 tobiasoed Exp $
  */
 
 
@@ -61,23 +61,25 @@ void scroll(XtPointer data);
 #define makestring(N) #N
 #define string(N) makestring(N)
 
+#define Min(a,b) ((a < b) ? (a) : (b))
+#define Max(a,b) ((a > b) ? (a) : (b))
+
 static String fallback[] = {
-	"Select-drag*matrix.rows:		" string(N_ROWS),
-	"Select-drag*matrix.columns:		" string(N_COLUMNS),
-	"Select-drag*matrix.visibleRows:	" string(N_VISIBLE),
-	"Select-drag*matrix.visibleColumns:	" string(N_VISIBLE),
-	"Select-drag*matrix.fixedRows:	" string(N_FIXED_ROWS),
-	"Select-drag*matrix.fixedColumns:	" string(N_FIXED_ROWS),
-/*	"Select-drag*matrix.trailingFixedColumns:	5",	*/
-	"Select-drag*matrix.gridType:	grid_cell_shadow",
+        "Select-drag*matrix.rows:                " string(N_ROWS),
+        "Select-drag*matrix.columns:             " string(N_COLUMNS),
+        "Select-drag*matrix.visibleRows:         " string(N_VISIBLE),
+        "Select-drag*matrix.visibleColumns:      " string(N_VISIBLE),
+        "Select-drag*matrix.fixedRows:           " string(N_FIXED_ROWS),
+        "Select-drag*matrix.fixedColumns:        " string(N_FIXED_ROWS),
+        "Select-drag*matrix.gridType:        grid_cell_shadow",
 /*
  * We have an uneditable matrix.
  */
-	"Select-drag*matrix.traversalOn:	False",
- 	"Select-drag*matrix.translations:	#override\\n"
- 	"			<Btn1Down>: SelectCell()\\n",
-	"Select-drag*matrix.selectionPolicy:	extended_select",
-	NULL
+        "Select-drag*matrix.traversalOn:        False",
+        "Select-drag*matrix.translations:        #override\\n"
+        "                        <Btn1Down>: SelectCell(PointerExtend)\\n",
+        "Select-drag*matrix.selectionPolicy:        extended_select",
+        NULL
 };
 
 /*
@@ -92,27 +94,28 @@ main(int argc, char *argv[])
     XtAppContext app;
 
     toplevel = XtAppInitialize(
-	&app, "Select-drag", NULL, 0, &argc, argv, fallback, NULL, 0);
+        &app, "Select-drag", NULL, 0, &argc, argv, fallback, NULL, 0);
 #ifdef USE_EDITRES
     XtAddEventHandler(toplevel, (EventMask)0, True,
                        _XEditResCheckMessages, NULL);
 #endif
 
     pane = XtVaCreateManagedWidget(
-	"pane", xmPanedWindowWidgetClass, toplevel,
-	NULL);
+        "pane", xmPanedWindowWidgetClass, toplevel,
+        NULL);
 
     pb = XtVaCreateManagedWidget(
-	"Reset", xmPushButtonWidgetClass, pane,
-	NULL);
+        "Reset", xmPushButtonWidgetClass, pane,
+        NULL);
 
     matrix = XtVaCreateManagedWidget(
-	"matrix", xbaeMatrixWidgetClass, pane,
-	XmNtopAttachment, XmATTACH_FORM,
-	XmNbottomAttachment, XmATTACH_FORM,
-	XmNleftAttachment, XmATTACH_FORM,
-	XmNtopAttachment, XmATTACH_FORM,
-	NULL);
+        "matrix", xbaeMatrixWidgetClass, pane,
+        XmNtopAttachment, XmATTACH_FORM,
+        XmNbottomAttachment, XmATTACH_FORM,
+        XmNleftAttachment, XmATTACH_FORM,
+        XmNtopAttachment, XmATTACH_FORM,
+        XmNselectScrollVisible, False,
+        NULL);
 
     XtAddCallback(matrix, XmNselectCellCallback, (XtCallbackProc)SelectCB, NULL);
 
@@ -145,36 +148,35 @@ SelectCB(Widget w, XtPointer client_data, XbaeMatrixSelectCellCallbackStruct *ca
      */
     /*XtVaSetValues(w, XmNselectScrollVisible, False, NULL);*/
 
-    if (call_data->num_params == 0)
+    if (strcmp(call_data->params[0], "PointerExtend") == 0)
     {
-	start_row = last_row = call_data->row;
-	start_column = last_column = call_data->column;
-	XbaeMatrixSelectCell(w, last_row, last_column);
+        start_row = last_row = call_data->row;
+        start_column = last_column = call_data->column;
+        XbaeMatrixSelectCell(w, last_row, last_column);
     }
     /*
      * Else we might be in an extended drag operation
      */
-    else if (!strcmp(call_data->params[ 0 ], "extend"))
+    else if (strcmp(call_data->params[ 0 ], "Extend") == 0)
     {
-	/*
-	 * This is a pretty simplistic, stupid implementation
-	 * of selecting a rectangle.
-	 */
-	if (call_data->row > last_row) {
-	    for (i = last_row + 1; i <= call_data->row; i++)
-		for (j = start_column; j <= call_data->column; j++)
-		    XbaeMatrixSelectCell(w, i, j);
-	}
-	if (call_data->column > last_column) {
-	    for (i = start_row; i <= last_row; i++)
-		for (j = last_column + 1; j <= call_data->column; j++)
-		    XbaeMatrixSelectCell(w, i, j);
-	}
-	last_row = call_data->row;
-	last_column = call_data->column;
-printf("%d, %d\n", last_row, last_column);
+        /*
+         * This is a pretty simplistic, stupid implementation
+         * of selecting a rectangle.
+         */
+
+        for (i = Min(start_row, last_row); i <= Max(start_row, last_row); i++)
+            for (j = Min(start_column, last_column); j <= Max(start_column, last_column); j++)
+                XbaeMatrixDeselectCell(w, i, j);
+        for (i = Min(start_row, call_data->row); i <= Max(start_row, call_data->row); i++)
+            for (j = Min(start_column, call_data->column); j <= Max(start_column, call_data->column); j++)
+                XbaeMatrixSelectCell(w, i, j);
+    
+        last_row = call_data->row;
+        last_column = call_data->column;
+        #if 0
+        printf("%d, %d\n", last_row, last_column);
+        #endif
     }
-    /*XtVaSetValues(w, XmNselectScrollVisible, True, NULL);*/
 }
 
 /* ARGSUSED */
@@ -187,27 +189,27 @@ LoadMatrix(Widget matrix)
     String **cells;
 
     XtVaGetValues(matrix,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  NULL);
+                  XmNrows, &rows,
+                  XmNcolumns, &columns,
+                  NULL);
 
     cells = (String **) XtMalloc(sizeof(String *) * rows);
     for (i = 0; i < rows; i++) {
-	cells[i] = (String *) XtMalloc(sizeof(String) * columns);
-	for (j = 0; j < columns; j++) {
-	    sprintf(buf, "r%dc%d", i, j);
-	    cells[i][j] = XtNewString(buf);
-	}
+        cells[i] = (String *) XtMalloc(sizeof(String) * columns);
+        for (j = 0; j < columns; j++) {
+            sprintf(buf, "r%dc%d", i, j);
+            cells[i][j] = XtNewString(buf);
+        }
     }
 
     XtVaSetValues(matrix,
-		  XmNcells, cells,
-		  NULL);
+                  XmNcells, cells,
+                  NULL);
 
     for (i = 0; i < rows; i++) {
-	for (j = 0; j < columns; j++)
-	    XtFree((XtPointer)cells[i][j]);
-	XtFree((XtPointer)cells[i]);
+        for (j = 0; j < columns; j++)
+            XtFree((XtPointer)cells[i][j]);
+        XtFree((XtPointer)cells[i]);
     }
     XtFree((XtPointer)cells);
 }
@@ -289,6 +291,6 @@ scroll(XtPointer data)
     }
 
     XtAppAddTimeOut(
-	    XtWidgetToApplicationContext(mw), 200, (XtTimerCallbackProc)scroll,
-	    (XtPointer)mw);
+            XtWidgetToApplicationContext(mw), 200, (XtTimerCallbackProc)scroll,
+            (XtPointer)mw);
 }
