@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/xbae/Xbae/examples/tests/0rows.c,v 1.1 2004/11/02 23:22:41 dannybackx Exp $
+/* $Header: /cvsroot/xbae/Xbae/examples/tests/0rows.c,v 1.2 2005/03/28 17:47:33 tobiasoed Exp $
  * if a table is initialized, filled with some rows, cleared to 0 rows (is empty) 
  * and then realized and mapped, a sigsev error occures,
  *
@@ -34,7 +34,7 @@ main(int argc, char *argv[])
 			XmNrows, 0,
 			XmNcolumns, 0,
 			XmNcolumnLabels, 0,
-			XmNXmColumnLabels, 0,
+			XmNxmColumnLabels, 0,
 			XmNcolumnMaxLengths, 0,
 			XmNcolumnAlignments, 0,
 			XmNcolumnFontBold, 0,
