@@ -1,5 +1,5 @@
 /*
- * $Id: manycellwidgets.c,v 1.4 2005/03/15 18:15:20 tobiasoed Exp $
+ * $Id: manycellwidgets.c,v 1.5 2005/12/22 16:59:48 tobiasoed Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,15 +47,32 @@ void CreateRow(Widget mat, int row)
 	Widget	toggle, l1, l2;
 	char	n[20];
 
-    static char default_text_translations[] = "#override\n"
-        "Ctrl <Key>osfUp                    :   EditCell(Up)\n"
-        "Ctrl <Key>osfDown                  :   EditCell(Down)\n"
-        "Ctrl <Key>osfLeft                  :   EditCell(Left)\n"
-        "Ctrl <Key>osfRight                 :   EditCell(Right)\n"
-        "<Key>osfPageDown                   :   PageDown()\n"
-        "<Key>osfPageUp                     :   PageUp()\n";
+    static char toggle_trans[] = 
+        "<Btn1Down>                         : DefaultAction() Arm()\n"
+        "<Btn1Up>                           : DefaultAction() Select() Disarm()\n"
+        "Ctrl <Key>osfUp                    : EditCell(Up)\n"
+        "Ctrl <Key>osfDown                  : EditCell(Down)\n"
+        "Ctrl <Key>osfLeft                  : EditCell(Left)\n"
+        "Ctrl <Key>osfRight                 : EditCell(Right)\n"
+        "<Key>osfPageDown                   : PageDown()\n"
+        "<Key>osfPageUp                     : PageUp()\n";
+    XtTranslations toggle_translations = XtParseTranslationTable(toggle_trans);
 
-    XtTranslations text_translations = XtParseTranslationTable(default_text_translations);
+    static char label_trans[] = 
+        "<Btn1Down>                         : DefaultAction()\n"
+        "<Btn1Up>                           : DefaultAction()\n";
+    XtTranslations label_translations = XtParseTranslationTable(label_trans);
+
+    static char text_trans[] = 
+        "<Btn1Down>                         : DefaultAction() grab-focus()\n"
+        "<Btn1Up>                           : DefaultAction()\n"
+        "Ctrl <Key>osfUp                    : EditCell(Up)\n"
+        "Ctrl <Key>osfDown                  : EditCell(Down)\n"
+        "Ctrl <Key>osfLeft                  : EditCell(Left)\n"
+        "Ctrl <Key>osfRight                 : EditCell(Right)\n"
+        "<Key>osfPageDown                   : PageDown()\n"
+        "<Key>osfPageUp                     : PageUp()\n";
+    XtTranslations text_translations = XtParseTranslationTable(text_trans);
 
     if (row == 1) {
         String rows[][3] = {
@@ -75,13 +92,14 @@ void CreateRow(Widget mat, int row)
         sprintf(n, "toggle%d", row);
         toggle = XmCreateToggleButton(mat, n, NULL, 0);
         XtManageChild(toggle);
-        XtOverrideTranslations(toggle, text_translations);
+        XtOverrideTranslations(toggle, toggle_translations);
         XbaeMatrixSetCellWidget(mat, row, 0, toggle);
     }
 
 	sprintf(n, "img%d", row);
 	l1 = XmCreateLabel(mat, n, NULL, 0);
     XtManageChild(l1);
+    XtOverrideTranslations(l1, label_translations);
     XbaeMatrixSetCellWidget(mat, row, 2, l1);
 
 	sprintf(n, "lab%d", row);
@@ -93,6 +111,12 @@ void CreateRow(Widget mat, int row)
 	/* XbaeMatrixSetRowHeight(mat, row, 30); */
 }
 
+void cbDefault(Widget w, XtPointer client, XtPointer call) {
+    XbaeMatrixDefaultActionCallbackStruct *cbs =
+        (XbaeMatrixDefaultActionCallbackStruct*) call;
+    printf("Double click in cell %d,%d\n", cbs->row, cbs->column);
+}
+
 int main(int argc, char *argv[])
 {
     Widget toplevel, form, mw, text;
@@ -137,6 +161,8 @@ int main(int argc, char *argv[])
             XmNtopWidget,	text,
 		    NULL);
 
+    XtAddCallback(mw, XmNdefaultActionCallback, cbDefault, (XtPointer) text);
+
     for (i=0; i<5; i++)
 	    CreateRow(mw, i);
 
