@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Create.h,v 1.29 2005/06/13 13:36:59 tobiasoed Exp $
+ * $Id: Create.h,v 1.32 2006/01/11 18:10:48 tobiasoed Exp $
  */
 
 /*
@@ -38,6 +38,10 @@
 extern "C" {
 #endif
 
+
+void xbaeCopyMarginSize(Widget widget, int offset, XrmValue *value);
+void xbaeCopyThickness(Widget widget, int offset, XrmValue *value);
+void xbaeCopySpace(Widget widget, int offset, XrmValue *value);
 void xbaeCopyBackground(Widget, int, XrmValue *);
 void xbaeCopyForeground(Widget, int, XrmValue *);
 void xbaeCopyDoubleClick(Widget, int, XrmValue *);
@@ -73,10 +77,11 @@ void xbaeGetGridLineGC(XbaeMatrixWidget);
 void xbaeGetResizeTopShadowGC(XbaeMatrixWidget);
 void xbaeGetResizeBottomShadowGC(XbaeMatrixWidget);
 
-XtPointer xbaeFontListGetFont(XbaeMatrixWidget, XmFontList, XmStringTag, XmFontType *);
-void xbaeInitFontFromFontList(XbaeMatrixWidget mw, XmFontList font_list, XmStringTag tag, XbaeMatrixFontInfo *font);
+XmFontListEntry xbaeFontListGetEntry(XbaeMatrixWidget, XmFontList, XmStringTag);
+void xbaeInitFontFromFontList(XbaeMatrixWidget, XmFontList, XmStringTag, XbaeMatrixFontInfo *);
 #if XmVERSION >= 2
-void xbaeInitFontFromRenderTable(XbaeMatrixWidget, XmRenderTable, XmStringTag *, int, XbaeMatrixFontInfo *);
+XmRendition xbaeRenderTableGetRendition(XbaeMatrixWidget, XmRenderTable, XmStringTag);
+void xbaeInitFontFromRenderTable(XbaeMatrixWidget, XmRenderTable, XmStringTag, XbaeMatrixFontInfo *);
 #endif
 void xbaeInitFonts(XbaeMatrixWidget);
 
@@ -101,8 +106,7 @@ void xbaeRegisterDisplay(Widget w);
 
 void xbaeFill_WithEmptyValues_PerCell(XbaeMatrixWidget mw, XbaeMatrixPerCellRec *p);
 
-void xbaeFreePerCellEntity (XbaeMatrixWidget mw, int row, int column);
-void xbaeFreePerCellRow (XbaeMatrixWidget mw, int row);
+void xbaeFreePerCellEntity(XbaeMatrixWidget mw, int row, int column);
 void xbaeFreePerCell(XbaeMatrixWidget mw);
 
 #ifdef __cplusplus
