@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: MatrixP.h,v 1.76 2005/07/31 18:35:31 tobiasoed Exp $
+ * $Id: MatrixP.h,v 1.81 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 /*
@@ -131,6 +131,18 @@ typedef struct {
     XbaeMatrixSetColumnColorsProc	set_column_colors;
     XbaeMatrixSetCellColorProc		set_cell_color;
     XtPointer				extension;
+    
+    XrmQuark QPointer;
+    XrmQuark QLeft;
+    XrmQuark QRight;
+    XrmQuark QUp;
+    XrmQuark QDown;
+    XrmQuark QPageUp;
+    XrmQuark QPageDown;
+    XrmQuark QFocus;
+    XrmQuark QLoosingFocus;
+    XrmQuark default_qtag;
+    
 } XbaeMatrixClassPart;
 
 /*
@@ -146,7 +158,6 @@ typedef struct _XbaeMatrixClassRec {
 
 externalref XbaeMatrixClassRec xbaeMatrixClassRec;
 
-
 /*
  * Inheritance constants for set/get/edit methods
  */
@@ -231,7 +242,7 @@ typedef struct {
     Boolean scroll_select;              /* Public: flag to scroll a selected cell    */
     Boolean traverse_fixed;             /* Public: allow traversal to fixed cells?   */
     Boolean calc_cursor_position;       /* Public: calculate insert pos from click   */
-    Boolean text_child_is_mapped;       /* Private: Is the text child on top of a cell? */
+    Boolean text_field_is_mapped;       /* Private: Is the text field on top of a cell? */
     unsigned int disable_redisplay;     /* Private: disable redisplay counter      */
 
     Boolean show_arrows;                /* Public: sow arrows when text obscured?    */
@@ -317,8 +328,8 @@ typedef struct {
     Pixel selected_background;          /* Public: background for selected cells     */
     Pixel selected_foreground;          /* Public: foreground for selected cells     */
     Pixel scroll_background;            /* Public: bacground for scrollbar */
-    Pixel text_background;              /* Public: background for the text child   */
-    Boolean text_background_is_cell;    /* Public: background for the text child when text_background is undefined */
+    Pixel text_background;              /* Public: background for the text field   */
+    Boolean text_background_is_cell;    /* Public: background for the text field when text_background is undefined */
 
     Position underline_position;        /* Public: number of pixels below the text baseline */
 
@@ -351,7 +362,7 @@ typedef struct {
     XbaeMatrixFontInfo label_font;          /* Private: Cashed info on the font used to draw labels */
     XbaeMatrixFontInfo current_draw_font;   /* Private: Cashed info on the font currently in draw_gc */
     XrmQuark current_draw_qtag;             /* Private: The quarkified tag of the font in the draw_gc */
-    XrmQuark current_text_qtag;             /* Private: The quarkified tag of the rendition installed on the text child */
+    XrmQuark current_text_qtag;             /* Private: The quarkified tag of the rendition installed on the text field */
 
     /*
      * private state
@@ -367,22 +378,19 @@ typedef struct {
     int  row_label_baseline;        /* Private: baseline of row labels       */
     int  cell_baseline;             /* Private: baseline of text in each cell     */
 
-    Time last_click_time;               /* Private: Used to detect double clicks    */
-    int double_click_interval;          /* Public: interval between clicks      */
-    int last_column;                    /* Private: Used to detect double clicks    */
-    int last_row;                       /* Private: Used to detect double clicks    */
+    Time last_click_time;           /* Private: Used to detect double clicks    */
+    int double_click_interval;      /* Public: interval between clicks      */
+    int last_column;                /* Private: Used to detect double clicks    */
+    int last_row;                   /* Private: Used to detect double clicks    */
 
-    int prev_column;                    /* Private: Used to compare tracking callback */
-    int prev_row;                       /* Private: Used to compare tracking callback */
-
-    int  current_column;            /* Private: column of the text field      */
-    int  current_row;               /* Private: row of the text field      */
+    int prev_column;                /* Private: Used to compare tracking callback */
+    int prev_row;                   /* Private: Used to compare tracking callback */
 
     Widget text_field;              /* Private: the text field       */
     Widget horizontal_sb;           /* Private: the horizontal scrollbar      */
     Widget vertical_sb;             /* Private: the vertical scrollbar    */
 
-    Widget clip_window;             /* Private: the clips           */
+    Widget center_clip;             /* Private: the clips           */
     Widget left_clip;
     Widget right_clip;
     Widget top_clip;
@@ -418,6 +426,14 @@ typedef struct _XbaeMatrixRec {
     XbaeMatrixPart  matrix;
 } XbaeMatrixRec;
 
+/*
+ * Constraint resources
+ */
+typedef struct {
+    int row;
+    int column;
+} XbaeMatrixConstraintsRec;
+
 /*
  * End of array indicator for converters of strings
  */
