@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Methods.h,v 1.18 2005/03/30 04:25:09 tobiasoed Exp $
+ * $Id: Methods.h,v 1.19 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 
@@ -44,8 +44,8 @@ void xbaeResize(XbaeMatrixWidget);
 /*
  * New Matrix methods
  */
-void xbaeUpdateTextChild(XbaeMatrixWidget mw, Boolean updateValue);
-void xbaeUpdateTextChildFont(XbaeMatrixWidget mw, XrmQuark qtag);
+void xbaeUpdateTextField(XbaeMatrixWidget mw, Boolean updateValue);
+void xbaeUpdateTextFieldFont(XbaeMatrixWidget mw, XrmQuark qtag);
 
 void xbaeSetCell(XbaeMatrixWidget, int, int, const String, Boolean);
 void xbaeModifyVerifyCB(Widget, XtPointer, XtPointer);
