@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Utils.h,v 1.40 2005/03/30 04:25:09 tobiasoed Exp $
+ * $Id: Utils.h,v 1.45 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 /*
@@ -41,12 +41,6 @@
 extern "C" {
 #endif
 
-extern int xbaeRowClip(XbaeMatrixWidget mw, int row);
-extern int xbaeColumnClip(XbaeMatrixWidget mw, int column);
-extern int xbaeCellClip(XbaeMatrixWidget mw, int row, int column);
-
-Widget xbaeGetCellClip(XbaeMatrixWidget, int, int);
-
 void xbaeGetColumnPositions(XbaeMatrixWidget);
 void xbaeGetRowPositions(XbaeMatrixWidget);
 int xbaeCheckRowPosition(XbaeMatrixWidget mw, int row);
@@ -75,22 +69,23 @@ void xbaeComputeSize(XbaeMatrixWidget, Boolean, Boolean);
 int xbaeCalculateLabelMaxLength(XbaeMatrixWidget mw, String *labels, XmString *xmlabels, int n_labels);
 int xbaeCalculateLabelMaxLines(String *labels, XmString *xmlabels, int n_labels);
 
-Boolean xbaeEventToMatrixXY(XbaeMatrixWidget mw, XEvent *event, int *x, int *y);
-Boolean xbaeMatrixXtoColumn(XbaeMatrixWidget mw,int *x, int *column);
-Boolean xbaeMatrixYtoRow(XbaeMatrixWidget mw,int *y, int *row);
-Boolean xbaeMatrixXYToRowCol(XbaeMatrixWidget mw,
+XbaeMatrixWidget xbaeEventToMatrixWidget(Widget w, XEvent *event);
+XbaeMatrixWidget xbaeEventToMatrixXY(Widget w, XEvent *event, int *x, int *y);
+int xbaeEventToRowColumn(Widget w, XEvent * event, int *row, int *column, int *x, int *y);
+
+int xbaeMatrixXtoColumn(XbaeMatrixWidget mw,int *x, int *column);
+int xbaeMatrixYtoRow(XbaeMatrixWidget mw,int *y, int *row);
+int xbaeMatrixXYToRowCol(XbaeMatrixWidget mw,
     int *x, int *y, int *row,int *column);
 
 int xbaeColumnToMatrixX(XbaeMatrixWidget mw, int column);
 int xbaeRowToMatrixY(XbaeMatrixWidget mw, int row);
-void xbaeRowColToMatrixXY(XbaeMatrixWidget mw, int row, int column,
-			    int *widget_x, int *widget_y);
 Widget xbaeRowColToClipXY(XbaeMatrixWidget, int, int, int *, int *);
 
 void xbaeHideCellWidget(XbaeMatrixWidget mw, Widget cw);
 void xbaePositionCellWidget(XbaeMatrixWidget mw, int row, int column);
-void xbaeHideTextChild(XbaeMatrixWidget mw);
-void xbaePositionTextChild(XbaeMatrixWidget mw);
+void xbaeHideTextField(XbaeMatrixWidget mw);
+void xbaePositionTextField(XbaeMatrixWidget mw);
 void xbaeSetInitialFocus(XbaeMatrixWidget mw);
 
 void xbaeSaneRectangle(XbaeMatrixWidget mw, XRectangle *rect_p,int rs, int cs, int re, int ce);
@@ -98,11 +93,8 @@ void xbaeSaneRectangle(XbaeMatrixWidget mw, XRectangle *rect_p,int rs, int cs, i
 void xbaeObjectLock(Widget);
 void xbaeObjectUnlock(Widget);
 
-int _xbaeStrcasecmp(const char *s1, const char *s2);
-int _xbaeStrncasecmp(const char *s1, const char *s2, size_t count);
-
-void xbaeScrollRows(XbaeMatrixWidget, Boolean Left, int step);
-void xbaeScrollColumns(XbaeMatrixWidget, Boolean Up, int step);
+void xbaeScrollRows(XbaeMatrixWidget, int step);
+void xbaeScrollColumns(XbaeMatrixWidget, int step);
 
 #ifdef __cplusplus
 }
