@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/xbae/Xbae/include/XbaeDebug.h,v 1.11 2005/04/02 16:56:23 tobiasoed Exp $
+ * $Header: /cvsroot/xbae/Xbae/include/XbaeDebug.h,v 1.12 2005/12/19 21:29:30 tobiasoed Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 LessTif Development Team
@@ -102,7 +102,7 @@ const char *_XbaeDebugResizePolicy2String(int n);
 
 const char *_XbaeDebugScrollingPolicy2String(unsigned char r);
 const char *_XbaeDebugDialogStyle2String(int a);
-const char *_XbaeDebugShadowTypeToString(const char s);
+const char *_XbaeDebugShadowTypeToString(unsigned char s);
 
 /*
  * Debug printing functions
