@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Converters.c,v 1.16 2004/11/29 02:23:10 tobiasoed Exp $
+ * $Id: Converters.c,v 1.20 2005/04/03 16:41:40 tobiasoed Exp $
  */
 
 /*
@@ -67,6 +67,8 @@ XbaeCvtStringToStringArray(Display * dpy, XrmValuePtr args, Cardinal * num_args,
         static String *array;
         String start = from->addr;
 
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeCvtStringToStringArray\n"));
+
         if (*num_args != 0)
                 XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "cvtStringToStringArray",
                                 "wrongParameters", "XbaeMatrix",
@@ -201,7 +203,7 @@ XbaeStringArrayDestructor(XtAppContext app, XrmValuePtr to, XtPointer converter_
         String *array = *(String **) to->addr;
         String *entry;
 
-        fprintf(stderr, "XbaeStringArrayDestructor(%p)\n", array);
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeStringArrayDestructor\n"));
 
         if (array == NULL)
                 return;
@@ -223,6 +225,8 @@ XbaeCvtStringToCellTable(Display * dpy, XrmValuePtr args, Cardinal * num_args, X
         int k, count;
         XrmValue lfrom, lto;
 
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeCvtStringToCellTable\n"));
+
         if (*num_args != 0)
                 XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "cvtStringToCellTable",
                                 "wrongParameters", "XbaeMatrix",
@@ -307,7 +311,7 @@ XbaeStringCellDestructor(XtAppContext app, XrmValuePtr to, XtPointer converter_d
         String **array = *(String ***) to->addr;
         String **entry, *row;
 
-        fprintf(stderr, "XbaeStringCellDestructor(%p)\n", array);
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeStringCellDestructor\n"));
 
         if (array == NULL)
                 return;
@@ -323,7 +327,7 @@ XbaeStringCellDestructor(XtAppContext app, XrmValuePtr to, XtPointer converter_d
 
 /*
  * Convert a comma separated list of short ints to array of widths.
- * The array is terminated with BAD_WIDTH.
+ * The array is terminated with BAD_SIZE.
  */
 
 /* ARGSUSED */
@@ -336,6 +340,8 @@ XbaeCvtStringToWidthArray(Display * dpy, XrmValuePtr args, Cardinal * num_args,
         char *ch;
         int i, count;
 
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeCvtStringToWidthArray\n"));
+
         if (*num_args != 0)
                 XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "cvtStringToWidthArray",
                                 "wrongParameters", "XbaeMatrix",
@@ -368,7 +374,7 @@ XbaeCvtStringToWidthArray(Display * dpy, XrmValuePtr args, Cardinal * num_args,
                  */
                 array = (short *) XtMalloc((count + 1) * sizeof(short));
 
-                array[count] = BAD_WIDTH;
+                array[count] = BAD_SIZE;
 
                 for (i = 0; i < count; i++) {
 
@@ -377,8 +383,8 @@ XbaeCvtStringToWidthArray(Display * dpy, XrmValuePtr args, Cardinal * num_args,
                         /*
                          * Find the comma at the end of this short
                          */
-                        /* EMPTY */
-                        for (; *start != '\0' && *start != ','; start++);
+                        for (; *start != '\0' && *start != ','; start++)
+                                ;
                         start++;
                 }
         }
@@ -403,6 +409,8 @@ XbaeWidthArrayDestructor(XtAppContext app, XrmValuePtr to, XtPointer converter_d
 {
         short *array = *(short **) to->addr;
 
+        DEBUGOUT(_XbaeDebug0(__FILE__, NULL, "XbaeWidthArrayDestructor\n"));
+
         XtFree((char *) array);
 }
 
@@ -462,8 +470,8 @@ XbaeCvtStringToMaxLengthArray(Display * dpy, XrmValuePtr args, Cardinal * num_ar
                         /*
                          * Find the comma at the end of this int
                          */
-                        /* EMPTY */
-                        for (; *start != '\0' && *start != ','; start++);
+                        for (; *start != '\0' && *start != ','; start++)
+                                ;
                         start++;
                 }
         }
@@ -570,8 +578,6 @@ XbaeCvtStringToBooleanArray(Display * dpy, XrmValuePtr args, Cardinal * num_args
                  */
                 array = (Boolean *) XtMalloc((count + 1) * sizeof(Boolean));
 
-/*	array[count] = BAD_WIDTH;
-*/
                 for (i = 0; i < count; i++) {
                         /*
                          * Skip leading white space
@@ -596,8 +602,8 @@ XbaeCvtStringToBooleanArray(Display * dpy, XrmValuePtr args, Cardinal * num_args
                         /*
                          * Find the comma at the end of this short
                          */
-                        /* EMPTY */
-                        for (; *start != '\0' && *start != ','; start++);
+                        for (; *start != '\0' && *start != ','; start++)
+                                ;
                         start++;
                 }
         }
@@ -1060,6 +1066,56 @@ XbaeCvtStringToGridType(Display * dpy, XrmValuePtr args, Cardinal * num_args, Xr
         return True;
 }
 
+/* ARGSUSED */
+Boolean
+XbaeCvtStringToWrapType(Display * dpy, XrmValuePtr args, Cardinal * num_args, XrmValuePtr from,
+                        XrmValuePtr to, XtPointer * data)
+{
+        static unsigned char wrap_type;
+        String start = from->addr;
+
+        if (*num_args != 0)
+                XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "cvtStringToWrapType",
+                                "wrongParameters", "XbaeMatrix",
+                                "String to WrapType conversion needs no extra arguments", NULL,
+                                NULL);
+
+        /*
+         * User didn't provide enough space
+         */
+        if (to->addr != NULL && to->size < sizeof(unsigned char)) {
+                to->size = sizeof(unsigned char);
+                return False;
+        }
+        /*
+         * Skip leading white space
+         */
+        while (isspace(*start))
+                start++;
+
+        if (XbaeStringsAreEqual(start, "wrap_none", 9))
+                wrap_type = XbaeWrapNone;    
+        else if (XbaeStringsAreEqual(start, "wrap_continuous", 15))
+                wrap_type = XbaeWrapContinuous;
+        else if (XbaeStringsAreEqual(start, "wrap_word", 9))
+                wrap_type = XbaeWrapWord;
+        else {
+                XtDisplayStringConversionWarning(dpy, from->addr, XmRWrapType);
+                return False;
+        }
+        
+        /*
+         * Store our return value
+         */
+        if (to->addr == NULL)
+                to->addr = (XtPointer) & wrap_type;
+        else
+                *(unsigned char *) to->addr = wrap_type;
+        to->size = sizeof(unsigned char);
+
+        return True;
+}
+
 /* ARGSUSED */
 Boolean
 #ifdef __VMS
