@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Create.c,v 1.79 2005/07/31 18:35:31 tobiasoed Exp $
+ * $Id: Create.c,v 1.86 2006/01/19 18:43:47 tobiasoed Exp $
  */
 
 /*
@@ -51,6 +51,46 @@
 
 static Pixmap createInsensitivePixmap(XbaeMatrixWidget mw);
 
+static Boolean xbaeEanableThinThickness(Widget widget) {
+#if XmVersion >= 2
+        Boolean enable_thin_thickness;
+        Display *display = XtDisplay(widget);
+        Widget xmDisplay = XmGetXmDisplay(display);
+
+        XtVaGetValues(xmDisplay, 
+            XmNenableThinThickness, &enable_thin_thickness, 
+            NULL);
+
+        return enable_thin_thickness;
+#else
+        return False
+#endif
+}
+
+void xbaeCopyMarginSize(Widget widget, int offset, XrmValue *value)
+{
+        static const Dimension thin = 1;
+        static const Dimension thick = 3;
+
+        value->addr = (XtPointer) (xbaeEanableThinThickness(widget) ? &thin : &thick);
+}
+
+void xbaeCopyThickness(Widget widget, int offset, XrmValue *value)
+{
+        static const Dimension thin = 1;
+        static const Dimension thick = 2;
+
+        value->addr = (XtPointer) (xbaeEanableThinThickness(widget) ? &thin : &thick);
+}
+
+void xbaeCopySpace(Widget widget, int offset, XrmValue *value)
+{
+        static const Dimension thin = 2;
+        static const Dimension thick = 4;
+
+        value->addr = (XtPointer) (xbaeEanableThinThickness(widget) ? &thin : &thick);
+}
+
 void xbaeCopyBackground(Widget widget, int offset, XrmValue * value)
 {
         value->addr = (XtPointer) & (widget->core.background_pixel);
@@ -225,7 +265,7 @@ void xbaeCopyColumnShadowTypes(XbaeMatrixWidget mw)
 
 /**************************************************************************************************/
 
-static short *copySizes(Widget w, short *sizes, int n)
+static short *copySizes(Widget w, short default_size, short *sizes, int n)
 {
         short *copy = NULL;
         int i;
@@ -235,13 +275,13 @@ static short *copySizes(Widget w, short *sizes, int n)
                 copy = (short *) XtMalloc(n * sizeof(short));
                 for (i = 0; i < n; i++) {
                         if (bad) {
-                                copy[i] = 1;
-                        } else if (sizes[i] < 1) {
-                                copy[i] = 1;
+                                copy[i] = default_size;
+                        } else if (sizes[i] < 0) {
+                                copy[i] = default_size;
                                 bad = True;
                                 XtAppWarningMsg(XtWidgetToApplicationContext(w),
                                                 "copySizes", "tooShort", "XbaeMatrix",
-                                                "XbaeMatrix: Row width or column height array is too short or contains illigal values",
+                                                "XbaeMatrix: Row height or column width array is too short or contains illigal values",
                                                 NULL, 0);
                         } else {
                                 copy[i] = sizes[i];
@@ -255,14 +295,14 @@ static short *copySizes(Widget w, short *sizes, int n)
 void xbaeCopyColumnWidths(XbaeMatrixWidget mw)
 {
         xbaeObjectLock((Widget) mw);
-        mw->matrix.column_widths = copySizes((Widget) mw, mw->matrix.column_widths, mw->matrix.columns);
+        mw->matrix.column_widths = copySizes((Widget) mw, DEFAULT_COLUMN_WIDTH(mw), mw->matrix.column_widths, mw->matrix.columns);
         xbaeObjectUnlock((Widget) mw);
 }
 
 void xbaeCopyRowHeights(XbaeMatrixWidget mw)
 {
         xbaeObjectLock((Widget) mw);
-        mw->matrix.row_heights = copySizes((Widget) mw, mw->matrix.row_heights, mw->matrix.rows);
+        mw->matrix.row_heights = copySizes((Widget) mw, DEFAULT_ROW_HEIGHT(mw), mw->matrix.row_heights, mw->matrix.rows);
         xbaeObjectUnlock((Widget) mw);
 }
 
@@ -752,73 +792,156 @@ static void xbaeInitFontInfo(XtPointer fontp, XmFontType type, XbaeMatrixFontInf
         }
 }
 
-XtPointer xbaeFontListGetFont(XbaeMatrixWidget mw, XmFontList font_list, XmStringTag tag, XmFontType *type)
+XmFontListEntry xbaeFontListGetEntry(XbaeMatrixWidget mw, XmFontList font_list, XmStringTag tag)
 {
         XmFontContext context;
-        XmFontListEntry font_list_entry;
-        XtPointer fontp;
+        XmFontListEntry font_list_entry = NULL;
+
+        XmStringTag tags[] = {NULL, _MOTIF_DEFAULT_LOCALE, XmFONTLIST_DEFAULT_TAG};
+        int n_tags = sizeof tags/sizeof *tags;
+        int i;
 
         xbaeObjectLock((Widget) mw);
 
-        DEBUGOUT(_XbaeDebug(__FILE__, (Widget)mw, "xbaeFontListGetFont: Font from rendertable\n"));
+        tags[0] = tag;
+
+        DEBUGOUT(_XbaeDebug(__FILE__, (Widget)mw, "xbaeFontListGetEntry\n"));
 
         /*
-         * Get a font context so we can loop over the XmFontListEntries
+         * Find the first tag that exists in the font_list
          */
-        if (!XmFontListInitFontContext(&context, font_list)) {
-                XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
-                              "xbaeFontListGetFont", "badFont", "XbaeMatrix",
-                              "XbaeMatrix: XmFontListInitFontContext failed, bad fontList",
-                              NULL, 0);
-        }
-
-        /*
-         * Find the font_list_entry that has tag "tag"
-         */
-        for(  font_list_entry = XmFontListNextEntry(context) 
-            ; font_list_entry != NULL
-            ; font_list_entry = XmFontListNextEntry(context)) {
-                XmStringTag entry_tag;
-
-                entry_tag = XmFontListEntryGetTag(font_list_entry);
-                
-                if (strcmp(entry_tag, tag) == 0) {
-                        XtFree((XtPointer) entry_tag);
-                        break;
+        for(i = (tag == NULL); i < n_tags && font_list_entry == NULL; i++) {
+                /*
+                 * Get a font context so we can loop over the XmFontListEntries
+                 */
+                if (!XmFontListInitFontContext(&context, font_list)) {
+                        XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
+                                      "xbaeFontListGetEntry", "badFont", "XbaeMatrix",
+                                      "XbaeMatrix: XmFontListInitFontContext failed, bad fontList",
+                                      NULL, 0);
                 }
-                XtFree((XtPointer) entry_tag);
+
+                /*
+                 * Search the font list for an entry with a matching tag
+                 */
+                for(  font_list_entry = XmFontListNextEntry(context) 
+                    ; font_list_entry != NULL
+                    ; font_list_entry = XmFontListNextEntry(context)) {
+                        XmStringTag entry_tag;
+
+                        entry_tag = XmFontListEntryGetTag(font_list_entry);
+
+                        if (strcmp(entry_tag, tags[i]) == 0) {
+                                XtFree((XtPointer) entry_tag);
+                                break;
+                        }
+                        XtFree((XtPointer) entry_tag);
+                }
+
+                XmFontListFreeFontContext(context);
         }
         
-        if(font_list_entry == NULL) {
-                XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
-                              "xbaeFontListGetFont", "badFont", "XbaeMatrix",
-                              "XbaeMatrix: Couldn't find tag in fontList", NULL, 0);
+        if (font_list_entry == NULL) {
+                /*
+                 * No tag matches, fall back to the first font
+                 */
+                XtAppWarningMsg(XtWidgetToApplicationContext((Widget) mw),
+                              "xbaeFontListGetEntry", "badTag", "XbaeMatrix",
+                              "XbaeMatrix: Couldn't find tag in fontlist",
+                              NULL, 0);
+
+                if (!XmFontListInitFontContext(&context, font_list)) {
+                        XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
+                                      "xbaeFontListGetEntry", "badFont", "XbaeMatrix",
+                                      "XbaeMatrix: XmFontListInitFontContext failed, bad fontList",
+                                      NULL, 0);
+                }
+
+                font_list_entry = XmFontListNextEntry(context);
+                XmFontListFreeFontContext(context);
         }
 
-        fontp = XmFontListEntryGetFont(font_list_entry, type);
-
-        XmFontListFreeFontContext(context);
-        
         xbaeObjectUnlock((Widget) mw);
         
-        return fontp;
+        return font_list_entry;
 }
 
 void xbaeInitFontFromFontList(XbaeMatrixWidget mw, XmFontList font_list, XmStringTag tag, XbaeMatrixFontInfo *font)
 {
+        XmFontListEntry font_list_entry;
+        XtPointer fontp = NULL;
         XmFontType type;
-        XtPointer fontp;
 
         DEBUGOUT(_XbaeDebug(__FILE__, (Widget) mw, "InitFontFromFontlist\n"));
 
-        fontp = xbaeFontListGetFont(mw, font_list, tag, &type);
-        xbaeInitFontInfo(fontp, type, font);
+        font_list_entry = xbaeFontListGetEntry(mw, font_list, tag);
 
-        xbaeObjectUnlock((Widget) mw);
+        if (font_list_entry == NULL) {
+                XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
+                              "xbaeInitFontFromFontList", "badFont", "XbaeMatrix",
+                              "XbaeMatrix: No fontListEntry found", NULL, 0);
+        }
+        
+        fontp = XmFontListEntryGetFont(font_list_entry, &type);
+        
+        if (fontp == NULL) {
+                XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
+                              "xbaeInitFontFromFontList", "badFont", "XbaeMatrix",
+                              "XbaeMatrix: XmFontListEntryGetFont failed, bad fontListEntry", NULL, 0);
+        }
+        
+        xbaeInitFontInfo(fontp, type, font);
 }
 
 #if XmVERSION >= 2
-void xbaeInitFontFromRenderTable(XbaeMatrixWidget mw, XmRenderTable render_table, XmStringTag *tags, int n_tags, XbaeMatrixFontInfo *font)
+XmRendition xbaeRenderTableGetRendition(XbaeMatrixWidget mw, XmRenderTable render_table, XmStringTag tag) {
+
+        XmRendition rendition = NULL;
+
+        XmStringTag tags[] = {NULL, _MOTIF_DEFAULT_LOCALE, XmFONTLIST_DEFAULT_TAG};
+        int n_tags = sizeof tags/sizeof *tags;
+        int i;
+
+        xbaeObjectLock((Widget) mw);
+
+        tags[0] = tag;
+
+        DEBUGOUT(_XbaeDebug(__FILE__, (Widget)mw, "xbaeRenderTableGetRendition\n"));
+
+        /*
+         * Find the first tag that exists in the render_table
+         */
+        for (i = (tag == NULL); i < n_tags && rendition == NULL; i++) {
+                rendition = XmRenderTableGetRendition(render_table, tags[i]);
+        }
+
+        if (rendition == NULL) {
+                /*
+                 * No tag matches, fall back to the first rendition
+                 */
+                int i, n;
+                XmStringTag *existing_tags;
+                n = XmRenderTableGetTags(render_table, &existing_tags);
+
+                XtAppWarningMsg(XtWidgetToApplicationContext((Widget) mw),
+                              "xbaeRenderTableGetRendition", "badTag", "XbaeMatrix",
+                              "XbaeMatrix: Couldn't find tag in renderTable",
+                              NULL, 0);
+
+                if (n) {
+                        rendition = XmRenderTableGetRendition(render_table, existing_tags[0]);
+                }
+                
+                for(i = 0; i < n; i++) {
+                    XtFree(existing_tags[i]);
+                }
+                XtFree((XtPointer) existing_tags);
+        }
+        
+        return rendition;
+}
+
+void xbaeInitFontFromRenderTable(XbaeMatrixWidget mw, XmRenderTable render_table, XmStringTag tag, XbaeMatrixFontInfo *font)
 {
         XmRendition rendition = NULL;
         String fontname;
@@ -826,34 +949,18 @@ void xbaeInitFontFromRenderTable(XbaeMatrixWidget mw, XmRenderTable render_table
         XtPointer fontp;
         Arg args[10];
         int m = 0;
-        int i;
 
-        DEBUGOUT(_XbaeDebug(__FILE__, (Widget) mw,
-		"xbaeInitFontFromRenderTable(%p)\n", render_table));
-
-#if 0
-        {
-            int n;
-            XmStringTag *tags;
-            n = XmRenderTableGetTags(render_table, &tags);
-
-            for(i = 0; i < n; i++) {
-                fprintf(stderr,"tag[%d] = %s\n",r,tags[t]);
-            }
-        }
-#endif
+        DEBUGOUT(_XbaeDebug(__FILE__, (Widget) mw, "xbaeInitFontFromRenderTable(%p)\n", render_table));
 
         /*
          * Find the first tag that exists in the render_table
          */
-        for (i = 0; i < n_tags && rendition == NULL; i++) {
-                rendition = XmRenderTableGetRendition(render_table, tags[i]);
-        }
+        rendition = xbaeRenderTableGetRendition(mw, render_table, tag);
         
         if (rendition == NULL) {
                 XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
                               "xbaeInitFontFromRenderTable", "badFont", "XbaeMatrix",
-                              "XbaeMatrix: Couldn't find tag in renderTable", NULL, 0);
+                              "XbaeMatrix: No rendition found", NULL, 0);
         }
 
         XtSetArg(args[m], XmNfontName, &fontname); m++;
@@ -862,23 +969,14 @@ void xbaeInitFontFromRenderTable(XbaeMatrixWidget mw, XmRenderTable render_table
 
         XmRenditionRetrieve(rendition, args, m);
 
-#if 0
-        fprintf(stderr, "fontname[%s] = %s\n", tags[i], fontname != (XtPointer) XmAS_IS ? fontname : "XmAS_IS");
-        fprintf(stderr, "type[%s] = %s\n", tags[i], 
-                type ==  XmFONT_IS_FONT
-                ? "XmFONT_IS_FONT" 
-                : type ==  XmFONT_IS_FONTSET ? "XmFONT_IS_FONTSET" : "?");
-#endif
-
-        if (fontp != NULL && fontp != (XtPointer) XmAS_IS) {
-                xbaeInitFontInfo(fontp, type, font);
-        } else {
-                /* Tobias: FIXME we have a problem! */
+        if (fontp == NULL || fontp == (XtPointer) XmAS_IS) {
                 XtAppErrorMsg(XtWidgetToApplicationContext((Widget) mw),
                               "xbaeFontFromRenderTable", "badFont", "XbaeMatrix",
                               "XbaeMatrix: The specified tag has no font loaded", NULL, 0);
         }
 
+        xbaeInitFontInfo(fontp, type, font);
+
         XmRenditionFree(rendition);
 }
 #endif
@@ -886,24 +984,24 @@ void xbaeInitFontFromRenderTable(XbaeMatrixWidget mw, XmRenderTable render_table
 void xbaeInitFonts(XbaeMatrixWidget mw) {
         #if XmVERSION >= 2
         if (mw->matrix.render_table) {
-                XmStringTag tags[] = {_MOTIF_DEFAULT_LOCALE, XmFONTLIST_DEFAULT_TAG};
-                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, tags, sizeof tags/sizeof *tags, &mw->matrix.cell_font);
+                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, NULL, &mw->matrix.cell_font);
         } else 
         #endif    
         {
-                xbaeInitFontFromFontList(mw, mw->matrix.font_list, XmFONTLIST_DEFAULT_TAG, &mw->matrix.cell_font);
+                xbaeInitFontFromFontList(mw, mw->matrix.font_list, NULL, &mw->matrix.cell_font);
         }
 
         #if XmVERSION >= 2
         if (mw->matrix.render_table) {
-                XmStringTag tags[] = {"labels", _MOTIF_DEFAULT_LOCALE, XmFONTLIST_DEFAULT_TAG};
-                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, tags, sizeof tags/sizeof *tags, &mw->matrix.label_font);
+                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, "labels", &mw->matrix.label_font);
         } else 
         #endif
-        if (mw->matrix.label_font_list) {
-                xbaeInitFontFromFontList(mw, mw->matrix.label_font_list, XmFONTLIST_DEFAULT_TAG, &mw->matrix.label_font);
-        } else if (mw->matrix.font_list) {
-                xbaeInitFontFromFontList(mw, mw->matrix.font_list, XmFONTLIST_DEFAULT_TAG, &mw->matrix.label_font);
+        {
+                if (mw->matrix.label_font_list) {
+                        xbaeInitFontFromFontList(mw, mw->matrix.label_font_list, NULL, &mw->matrix.label_font);
+                } else if (mw->matrix.font_list) {
+                        xbaeInitFontFromFontList(mw, mw->matrix.font_list, "labels", &mw->matrix.label_font);
+                }
         }
 }
 
@@ -956,60 +1054,26 @@ void xbaeCreatePerCell(XbaeMatrixWidget mw)
         xbaeObjectUnlock((Widget) mw);
 }
 
-static void __FreePixmap(XbaeMatrixWidget mw, Pixmap * p)
-{
-        if (*p && (*p != XmUNSPECIFIED_PIXMAP)) {
-                XFreePixmap(XtDisplay(mw), *p);
-                *p = XmUNSPECIFIED_PIXMAP;
+void xbaeFreePerCellEntity(XbaeMatrixWidget mw, int row, int column) {
+        /* Unmanage, unmap any cell widgets that might be visible */
+        if (mw->matrix.per_cell[row][column].widget) {
+                XtUnmanageChild(mw->matrix.per_cell[row][column].widget);
         }
-}
-
-static void __FreeString(String * p)
-{
-        if (*p != NULL) {
-                XtFree(*p);
-                *p = NULL;
+        /* Free the cellValue string */
+        if (mw->matrix.per_cell[row][column].CellValue) {
+                XtFree(mw->matrix.per_cell[row][column].CellValue);
+                mw->matrix.per_cell[row][column].CellValue = NULL;
         }
-}
-
-void xbaeFreePerCellEntity(XbaeMatrixWidget mw, int row, int column)
-{
-	DEBUGOUT(_XbaeDebug(__FILE__, (Widget)mw, "xbaeFreePerCellEntity(%d,%d)\n", row, column));
-        /*
-         * CellValue: must be freed, alloc in xbae!
-         */
-        __FreeString(&mw->matrix.per_cell[row][column].CellValue);
-
-        /*
-         * user_data: must not be freed, user-defined
-         */
-
-        /*
-         * Pixmap: free it, alloc in xbae!
-         */
-        __FreePixmap(mw, &mw->matrix.per_cell[row][column].pixmap);
-
-        /*
-         * mask: free it, alloc in xbae!
-         */
-        __FreePixmap(mw, &mw->matrix.per_cell[row][column].mask);
-
-}
-
-void xbaeFreePerCellRow(XbaeMatrixWidget mw, int row)
-{
-        int j = 0;
-        if (mw->matrix.per_cell[row]) {
-                for (j = 0; j < mw->matrix.columns; j++)
-                        xbaeFreePerCellEntity(mw, row, j);
-                XtFree((char *) mw->matrix.per_cell[row]);
-                mw->matrix.per_cell[row] = NULL;
+        /* Update the number of cells that are selected */
+        if (mw->matrix.per_cell[row][column].selected) {
+                mw->matrix.num_selected_cells--;
         }
 }
 
 void xbaeFreePerCell(XbaeMatrixWidget mw)
 {
-        int i = 0;
+        int row;
+        int column;
 
         /*
          * Free each row of XtPointer pointers
@@ -1018,13 +1082,16 @@ void xbaeFreePerCell(XbaeMatrixWidget mw)
 
                 xbaeObjectLock((Widget) mw);
 
-                for (i = 0; i < mw->matrix.rows; i++)
-                        xbaeFreePerCellRow(mw, i);
+                for (row = 0; row < mw->matrix.rows; row++) {
+                        for (column = 0; column < mw->matrix.columns; column++) {
+                                xbaeFreePerCellEntity(mw, row, column);
+                        }
+                        XtFree((XtPointer) mw->matrix.per_cell[row]);
+                }
 
-                XtFree((char *) mw->matrix.per_cell);
+                XtFree((XtPointer) mw->matrix.per_cell);
                 mw->matrix.per_cell = NULL;
 
                 xbaeObjectUnlock((Widget) mw);
         }
-
 }
