@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/xbae/Xbae/src/DebugUtil.c,v 1.19 2005/04/02 16:56:24 tobiasoed Exp $
+ * $Header: /cvsroot/xbae/Xbae/src/DebugUtil.c,v 1.20 2005/12/19 14:59:32 tobiasoed Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 LessTif Development Team 
@@ -24,7 +24,7 @@
  **/
 
 static const char rcsid[] =
-    "$Header: /cvsroot/xbae/Xbae/src/DebugUtil.c,v 1.19 2005/04/02 16:56:24 tobiasoed Exp $";
+    "$Header: /cvsroot/xbae/Xbae/src/DebugUtil.c,v 1.20 2005/12/19 14:59:32 tobiasoed Exp $";
 
 #include "XbaeConfig.h"
 
@@ -32,6 +32,7 @@ static const char rcsid[] =
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include <strings.h>
 #include <signal.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -276,6 +277,33 @@ static const struct
 
 /**************************************************************************************************/
 
+#ifndef XBAE_PRODUCTION
+/* strcasecmp() is not always available */
+static int xbaeStrcasecmp(const char *s1, const char *s2)
+{
+#ifdef HAVE_STRNCASECMP
+        return strcasecmp(s1, s2);
+#else
+#ifdef HAVE_STRNICMP
+        return stricmp(s1, s2);
+#else
+        int c1, c2;
+
+        while (*s1 && *s2) {
+                c1 = tolower(*s1);
+                c2 = tolower(*s2);
+                if (c1 != c2) {
+                        return (c1 - c2);
+                }
+                s1++;
+                s2++;
+        }
+        return (int) (*s1 - *s2);
+#endif
+#endif
+}
+#endif
+
 #ifndef XBAE_PRODUCTION
 /* catches the signal defined by XBAE_DEBUG_SIGNAL and toggles the
    global debugging flag.
@@ -309,7 +337,7 @@ static Boolean siginstall(void)
 #ifdef DEBUGSIG
         fprintf(stderr, "siginstall(): trying to catch %s\n", ptr);
 #endif
-        if ((*ptr == '\0') || (_xbaeStrcasecmp(ptr, "none") == 0)) {
+        if ((*ptr == '\0') || (xbaeStrcasecmp(ptr, "none") == 0)) {
             fprintf(stderr, "siginstall(): empty value for XBAE_DEBUG_SIGNAL\n");
         }
 #if defined(SIGBREAK)
@@ -457,9 +485,9 @@ static Boolean ValidateSource(const char *fn)
            search is later on required */
 
         /* list does not exist, is empty or set to "none" */
-        if (sourcelist == NULL || sourcelist[0] =='\0' || (_xbaeStrcasecmp(sourcelist, "none") == 0)) {
+        if (sourcelist == NULL || sourcelist[0] =='\0' || (xbaeStrcasecmp(sourcelist, "none") == 0)) {
             flag_none = True;
-        } else if (_xbaeStrcasecmp(sourcelist, "all") == 0) {
+        } else if (xbaeStrcasecmp(sourcelist, "all") == 0) {
             flag_all = True;
         } else {
             const char *s, *p;
@@ -1981,7 +2009,7 @@ extern const char * _LtDebugFrameChildType2String(int action)
     }
 }
 
-extern const char *_XbaeDebugShadowTypeToString(const char s)
+extern const char *_XbaeDebugShadowTypeToString(unsigned char s)
 {
     switch (s) {
     case XmSHADOW_IN:
