@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Draw.c,v 1.126 2005/07/31 18:35:32 tobiasoed Exp $
+ * $Id: Draw.c,v 1.129 2006/05/16 19:14:43 tobiasoed Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -677,10 +677,7 @@ xbaeDrawCellString(XbaeMatrixWidget mw, Window win,
 {
         GC gc = mw->matrix.draw_gc;
 
-        static XrmQuark default_qtag = NULLQUARK;
-        if (default_qtag == NULLQUARK) {
-            default_qtag = XrmUniqueQuark();
-        }
+        XrmQuark default_qtag = ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.default_qtag;
 
         if (   mw->matrix.current_draw_qtag == NULLQUARK
             || (qtag == NULLQUARK && mw->matrix.current_draw_qtag != default_qtag)
@@ -694,7 +691,7 @@ xbaeDrawCellString(XbaeMatrixWidget mw, Window win,
                         mw->matrix.current_draw_qtag = qtag;
                         #if XmVERSION >= 2
                         if (mw->matrix.render_table) {
-                                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, &tag , 1, &mw->matrix.current_draw_font);
+                                xbaeInitFontFromRenderTable(mw, mw->matrix.render_table, tag, &mw->matrix.current_draw_font);
                         } else 
                         #endif
                         {
@@ -967,7 +964,7 @@ void xbaeDrawCell(XbaeMatrixWidget mw, int row, int column)
 }
 
 void
-xbaeGetCellValues(XbaeMatrixWidget mw, int row, int column, Boolean textChild, XbaeMatrixCellValuesStruct *cell_values)
+xbaeGetCellValues(XbaeMatrixWidget mw, int row, int column, Boolean is_text_field, XbaeMatrixCellValuesStruct *cell_values)
 {
         assert(row >= 0 && row < mw->matrix.rows && column >= 0 && column < mw->matrix.columns);
 
@@ -1030,9 +1027,9 @@ xbaeGetCellValues(XbaeMatrixWidget mw, int row, int column, Boolean textChild, X
         }
 
 	    /*
-	     * Now adjust colors for selection or textChild
+	     * Now adjust colors for selection or is_text_field
          */
-        if (textChild) {
+        if (is_text_field) {
                 if (mw->matrix.text_background != XmUNSPECIFIED_PIXEL) {
                         cell_values->drawCB.background = mw->matrix.text_background;
                 } else if (!mw->matrix.text_background_is_cell) {
