@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Public.c,v 1.90 2005/06/06 19:50:22 dannybackx Exp $
+ * $Id: Public.c,v 1.97 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 /*
@@ -112,7 +112,7 @@ static XbaeMatrixWidget xbaeCheckClass(Widget w, const char *fcn) {
                 XtAppWarningMsg(XtWidgetToApplicationContext(w), 
                                 fcn, "badWidget", "XbaeMatrix", msg2, NULL, 0);
 
-        }            
+        }
         return mw;
 }
 
@@ -963,36 +963,25 @@ void XbaeMatrixDeunderlineColumn(Widget w, int column)
 
 /*
  * Help the user know what row & column he is in given an x & y (via an event).
- * Return True on success, False on failure.
+ * Return the clip region on success, 0 on failure.
  */
 int XbaeMatrixGetEventRowColumn(Widget w, XEvent * event, int *row, int *column)
 {
-        int x, y;
+        int x, y, clip_region;
         XbaeMatrixWidget mw;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
         if (mw == NULL) {
                 xbaeObjectUnlock(w);
-                return False;
-        }
-
-        /* Convert the event to the correct XY for the matrix widget. */
-        if (!xbaeEventToMatrixXY(mw, event, &x, &y)) {
-                xbaeObjectUnlock(w);
-                return False;
-        }
-
-        /* Convert the point to a row,column. If it does not pick a valid cell,
-           then return. */
-        if (!xbaeMatrixXYToRowCol(mw, &x, &y, row, column)) {
-                xbaeObjectUnlock(w);
-                return False;
+                return 0;
         }
+        
+        clip_region = xbaeEventToRowColumn(w, event, row, column, &x, &y);
 
         xbaeObjectUnlock(w);
         
-        return True;
+        return clip_region;
 }
 
 /*
@@ -1000,6 +989,7 @@ int XbaeMatrixGetEventRowColumn(Widget w, XEvent * event, int *row, int *column)
  */
 Boolean XbaeMatrixEventToXY(Widget w, XEvent * event, int *x, int *y)
 {
+        int row, column;
         XbaeMatrixWidget mw;
 
         xbaeObjectLock(w);
@@ -1009,9 +999,9 @@ Boolean XbaeMatrixEventToXY(Widget w, XEvent * event, int *x, int *y)
                 return False;
         }
 
-        if (!xbaeEventToMatrixXY(mw, event, x, y)) {
-                xbaeObjectUnlock(w);
-                return False;
+        if (xbaeEventToRowColumn(w, event, &row, &column, x, y)) {
+                *x += xbaeColumnToMatrixX(mw, column);
+                *y += xbaeRowToMatrixY(mw, row);
         }
 
         /* Tobias: FIXME: Using xbaeEventToMatrixXY instead of the old xbaeEventToXY
@@ -1054,6 +1044,7 @@ Boolean XbaeMatrixRowColToXY(Widget w, int row, int column, int *x, int *y)
 void XbaeMatrixGetCurrentCell(Widget w, int *row, int *column)
 {
         XbaeMatrixWidget mw;
+        int current_row, current_column;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -1062,19 +1053,21 @@ void XbaeMatrixGetCurrentCell(Widget w, int *row, int *column)
                 return;
         }
 
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
+
         /* 
          * Make sure we return something valid. 
          * At least Grace depends on it... 
          */
 
-        if (mw->matrix.current_row >= 0 && mw->matrix.current_row < mw->matrix.rows) {
-                *row = mw->matrix.current_row;
+        if (current_row >= 0 && current_row < mw->matrix.rows) {
+                *row = current_row;
         } else {
                 *row = xbaeTopRow(mw);
         }
 
-        if (mw->matrix.current_column >= 0 && mw->matrix.current_column < mw->matrix.columns) {
-                *column = mw->matrix.current_column;
+        if (current_column >= 0 && current_column < mw->matrix.columns) {
+                *column = current_column;
         } else {
                 *column = xbaeLeftColumn(mw);
         }
@@ -1096,8 +1089,7 @@ void XbaeMatrixSetCurrentCellPosition(Widget w, int row, int column)
                 return;
         }
 
-        mw->matrix.current_row = row;
-        mw->matrix.current_column = column;
+        XtVaSetValues(TextField(mw), XmNattachRow, row, XmNattachColumn, column, NULL);
 
         xbaeObjectUnlock(w);
 }
@@ -1122,8 +1114,8 @@ void XbaeMatrixRefresh(Widget w)
         if (!mw->matrix.disable_redisplay && XtIsRealized(w)) {
                 xbaeRefresh(mw, True);
 
-                if (mw->matrix.text_child_is_mapped) {
-                        xbaeUpdateTextChild(mw, True);
+                if (mw->matrix.text_field_is_mapped) {
+                        xbaeUpdateTextField(mw, True);
                 }
         }
 
@@ -1136,6 +1128,7 @@ void XbaeMatrixRefresh(Widget w)
 void XbaeMatrixRefreshCell(Widget w, int row, int column)
 {
         XbaeMatrixWidget mw;
+        int current_row, current_column;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -1148,10 +1141,12 @@ void XbaeMatrixRefreshCell(Widget w, int row, int column)
                 xbaeDrawCell(mw, row, column);
         }
 
-        if(mw->matrix.text_child_is_mapped 
-           && row == mw->matrix.current_row 
-           && column == mw->matrix.current_column) {
-                xbaeUpdateTextChild(mw, True);
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
+
+        if(mw->matrix.text_field_is_mapped 
+           && row == current_row 
+           && column == current_column) {
+                xbaeUpdateTextField(mw, True);
         }
 
         xbaeObjectUnlock(w);
@@ -1164,6 +1159,7 @@ void XbaeMatrixRefreshColumn(Widget w, int column)
 {
         int row, tr, br;
         XbaeMatrixWidget mw;
+        int current_column;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -1182,8 +1178,10 @@ void XbaeMatrixRefreshColumn(Widget w, int column)
                 }
         }
 
-        if(mw->matrix.text_child_is_mapped && column == mw->matrix.current_column) {
-                xbaeUpdateTextChild(mw, True);
+        XtVaGetValues(TextField(mw), XmNattachColumn, &current_column, NULL);
+
+        if(mw->matrix.text_field_is_mapped && column == current_column) {
+                xbaeUpdateTextField(mw, True);
         }
 
         xbaeObjectUnlock(w);
@@ -1196,6 +1194,7 @@ void XbaeMatrixRefreshRow(Widget w, int row)
 {
         int column, lc, rc;
         XbaeMatrixWidget mw;
+        int current_row;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -1214,8 +1213,10 @@ void XbaeMatrixRefreshRow(Widget w, int row)
                 }
         }
 
-        if(mw->matrix.text_child_is_mapped && row == mw->matrix.current_row) {
-                xbaeUpdateTextChild(mw, True);
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, NULL);
+
+        if(mw->matrix.text_field_is_mapped && row == current_row) {
+                xbaeUpdateTextField(mw, True);
         }
 
         xbaeObjectUnlock(w);
@@ -1476,11 +1477,14 @@ void XbaeMatrixSetCellWidget(Widget w, int row, int column, Widget widget)
                          */
                         if (widget) {
                                 /*
-                                 * If the textChild happens to be in this cell, hide it
+                                 * If the TextField happens to be in this cell, hide it
                                  */
-                                if (   mw->matrix.text_child_is_mapped
-                                    && row == mw->matrix.current_row && column == mw->matrix.current_column) {
-                                        xbaeHideTextChild(mw);
+                                int current_row, current_column;
+                                XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
+                                
+                                if (   mw->matrix.text_field_is_mapped
+                                    && row == current_row && column == current_column) {
+                                        xbaeHideTextField(mw);
                                 }
 
                                 /*
@@ -1548,6 +1552,8 @@ void XbaeMatrixSetCellTag(Widget w, int row, int column, XmStringTag tag)
         }
 
         if (mw->matrix.per_cell[row][column].qtag != qtag) {
+                int current_row, current_column;
+                XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
 
                 mw->matrix.per_cell[row][column].qtag = qtag;
 
@@ -1555,16 +1561,16 @@ void XbaeMatrixSetCellTag(Widget w, int row, int column, XmStringTag tag)
                         xbaeDrawCell(mw, row, column);
                 }
 
-                if (   mw->matrix.text_child_is_mapped
-                    && row == mw->matrix.current_row && column == mw->matrix.current_column) {
+                if (   mw->matrix.text_field_is_mapped
+                    && row == current_row && column == current_column) {
                         /*
-                         * Update the font the textChild uses
+                         * Update the font the TextField uses
                          */
-                        xbaeUpdateTextChildFont(mw, qtag);
+                        xbaeUpdateTextFieldFont(mw, qtag);
                         /*
-                         * we need to reset the size of the text child as the above has adverse effects
+                         * we need to reset the size of the TextField as the above has adverse effects
                          */
-                        xbaePositionTextChild(mw);
+                        xbaePositionTextField(mw);
                 }
         }
         
@@ -2702,6 +2708,7 @@ void XbaeMatrixSetXmRowLabel(Widget w, int row, XmString value)
 void XbaeMatrixSetColumnWidth(Widget w, int column, int width)
 {
         XbaeMatrixWidget mw;
+        int current_column;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -2710,12 +2717,24 @@ void XbaeMatrixSetColumnWidth(Widget w, int column, int width)
                 return;
         }
 
-        if (width < 1) {
+        if (width < 0) {
                 mw->matrix.column_widths[column] = DEFAULT_COLUMN_WIDTH(mw);
         } else {
                 mw->matrix.column_widths[column] = width;
         }
         
+        /*
+         * Hide the TextField if the cell that was edited was hidden
+         */
+        XtVaGetValues(TextField(mw), XmNattachColumn, &current_column, NULL);
+        if (mw->matrix.column_widths[current_column] == 0) {
+                XtVaSetValues(TextField(mw), XmNattachRow, -1, XmNattachColumn, -1, NULL);
+
+                if (mw->matrix.text_field_is_mapped) {
+                        xbaeHideTextField(mw);
+                }
+        }
+
         xbaeGetColumnPositions(mw);
 
         if (!mw->matrix.disable_redisplay && XtIsRealized(w)) {
@@ -2747,6 +2766,7 @@ int XbaeMatrixGetColumnWidth(Widget w, int column)
 void XbaeMatrixSetRowHeight(Widget w, int row, int height)
 {
         XbaeMatrixWidget mw;
+        int current_row;
 
         xbaeObjectLock(w);
         mw = XBAE_CHECK_CLASS(w);
@@ -2755,12 +2775,24 @@ void XbaeMatrixSetRowHeight(Widget w, int row, int height)
                 return;
         }
 
-        if (height < 1) {
+        if (height < 0) {
                 mw->matrix.row_heights[row] = DEFAULT_ROW_HEIGHT(mw);
         } else {
                 mw->matrix.row_heights[row] = height;
         }
 
+        /*
+         * Hide the TextField if the cell that was edited was hidden
+         */
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, NULL);
+        if (mw->matrix.row_heights[current_row] == 0) {
+                XtVaSetValues(TextField(mw), XmNattachRow, -1, XmNattachColumn, -1, NULL);
+
+                if (mw->matrix.text_field_is_mapped) {
+                        xbaeHideTextField(mw);
+                }
+        }
+
         xbaeGetRowPositions(mw);
 
         if (!mw->matrix.disable_redisplay && XtIsRealized(w)) {
