@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: ScrollMgr.c,v 1.78 2005/04/15 21:43:49 tobiasoed Exp $
+ * $Id: ScrollMgr.c,v 1.82 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,6 +55,9 @@ void xbaeScrollVertCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct
 {
         XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
         int delta = VERT_ORIGIN(mw) - call_data->value;
+        int current_row;
+
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, NULL);
 
         if (delta == 0) {
                 /* Didn't scroll */
@@ -69,8 +72,8 @@ void xbaeScrollVertCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct
         /*
          * The textField needs to scroll along with the cells.
          */
-        if (mw->matrix.text_child_is_mapped && !IS_FIXED_ROW(mw, mw->matrix.current_row)) {
-                xbaePositionTextChild(mw);
+        if (mw->matrix.text_field_is_mapped && !IS_FIXED_ROW(mw, current_row)) {
+                xbaePositionTextField(mw);
         }
         
         /* Cell widgets that are in the vertical scrolling region
@@ -93,8 +96,8 @@ void xbaeScrollVertCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct
          * Scroll the contents of the clips
          */
 
-        if (XtIsManaged(ClipChild(mw)))
-                XbaeClipScrollVert(ClipChild(mw), mw->matrix.draw_gc, delta);
+        if (XtIsManaged(CenterClip(mw)))
+                XbaeClipScrollVert(CenterClip(mw), mw->matrix.draw_gc, delta);
         
         if (XtIsManaged(LeftClip(mw)))
                 XbaeClipScrollVert(LeftClip(mw), mw->matrix.draw_gc, delta);
@@ -115,6 +118,9 @@ void xbaeScrollHorizCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruc
 {
         XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
         int delta = HORIZ_ORIGIN(mw) - call_data->value;
+        int current_column;
+
+        XtVaGetValues(TextField(mw), XmNattachColumn, &current_column, NULL);
 
         if (delta == 0) {
                 /* Didn't scroll */
@@ -129,8 +135,8 @@ void xbaeScrollHorizCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruc
         /*
          * The textField needs to scroll along with the cells.
          */
-        if (mw->matrix.text_child_is_mapped && !IS_FIXED_COLUMN(mw, mw->matrix.current_column)) {
-                xbaePositionTextChild(mw);
+        if (mw->matrix.text_field_is_mapped && !IS_FIXED_COLUMN(mw, current_column)) {
+                xbaePositionTextField(mw);
         }
 
         /* Cell widgets that are in the horizontal scrolling region
@@ -153,8 +159,8 @@ void xbaeScrollHorizCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruc
          * Scroll the contents of the clips
          */
 
-        if (XtIsManaged(ClipChild(mw)))
-                XbaeClipScrollHoriz(ClipChild(mw), mw->matrix.draw_gc, delta);
+        if (XtIsManaged(CenterClip(mw)))
+                XbaeClipScrollHoriz(CenterClip(mw), mw->matrix.draw_gc, delta);
 
         if (XtIsManaged(TopClip(mw)))
                 XbaeClipScrollHoriz(TopClip(mw), mw->matrix.draw_gc, delta);
@@ -312,7 +318,7 @@ void xbaeRedrawAll(XbaeMatrixWidget mw, int rs, int cs, int re, int ce)
         
         XRectangle rect;
         Widget clips[] = {
-                ClipChild(mw),
+                CenterClip(mw),
                 TopClip(mw),
                 LeftClip(mw),
                 RightClip(mw),
@@ -368,8 +374,8 @@ void xbaeRefresh(XbaeMatrixWidget mw, Boolean relayout)
 
         XClearArea(XtDisplay((Widget) mw), XtWindow((Widget) mw), 0, 0, 0, 0, True);
 
-        if (XtIsManaged(ClipChild(mw)))
-                XClearArea(XtDisplay(mw), XtWindow(ClipChild(mw)), 0, 0, 0, 0, True);
+        if (XtIsManaged(CenterClip(mw)))
+                XClearArea(XtDisplay(mw), XtWindow(CenterClip(mw)), 0, 0, 0, 0, True);
         if (XtIsManaged(TopClip(mw)))
                 XClearArea(XtDisplay(mw), XtWindow(TopClip(mw)), 0, 0, 0, 0, True);
         if (XtIsManaged(LeftClip(mw)))
