@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Utils.c,v 1.112 2005/07/12 09:13:30 dannybackx Exp $
+ * $Id: Utils.c,v 1.123 2006/05/16 19:59:53 tobiasoed Exp $
  */
 
 /*
@@ -52,7 +52,7 @@
 
 #include <XbaeDebug.h>
 
-int xbaeRowClip(XbaeMatrixWidget mw, int row)
+static int xbaeRowClip(XbaeMatrixWidget mw, int row)
 {
         int clip;
 
@@ -69,7 +69,7 @@ int xbaeRowClip(XbaeMatrixWidget mw, int row)
         return clip;
 }
 
-int xbaeColumnClip(XbaeMatrixWidget mw, int column)
+static int xbaeColumnClip(XbaeMatrixWidget mw, int column)
 {
         int clip;
 
@@ -86,7 +86,7 @@ int xbaeColumnClip(XbaeMatrixWidget mw, int column)
         return clip;
 }
 
-int xbaeCellClip(XbaeMatrixWidget mw, int row, int column)
+static int xbaeCellClip(XbaeMatrixWidget mw, int row, int column)
 {
         return xbaeRowClip(mw, row) | xbaeColumnClip(mw, column);
 }
@@ -95,15 +95,15 @@ int xbaeCellClip(XbaeMatrixWidget mw, int row, int column)
  * Returns the widget to which a cell belongs, i.e. the matrix, or one
  * or one of the extra clips which handle the fixed row/col cells/labels
  */
-Widget xbaeGetCellClip(XbaeMatrixWidget mw, int row, int column)
+static Widget xbaeGetCellClip(XbaeMatrixWidget mw, int row, int column)
 {
         int clip = xbaeCellClip(mw, row, column);
         Widget w;
 
         switch (clip) {
         case CLIP_VISIBLE_WIDTH | CLIP_VISIBLE_HEIGHT:
-                /* not fixed at all - on clip child */
-                w = ClipChild(mw);
+                /* not fixed at all - on center clip */
+                w = CenterClip(mw);
                 break;
 
         case CLIP_FIXED_COLUMNS | CLIP_VISIBLE_HEIGHT:
@@ -155,12 +155,16 @@ xbaeGetPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int
         int i, pos;
         
         if (size_in_pixels) {
-                for (i = 0, pos = 0; i < n; pos += sizes[i], i++) {
+                for (i = 0, pos = 0; i < n; i++) {
                         positions[i] = pos;
+                        pos += sizes[i];
                 }
         } else {
-                for (i = 0, pos = 0; i < n; pos += sizes[i] * font_size + 2 * border_size, i++) {
+                for (i = 0, pos = 0; i < n; i++) {
                         positions[i] = pos;
+                        if (sizes[i] != 0) {
+                                pos += sizes[i] * font_size + 2 * border_size;
+                        }
                 }
         }
         /* Tobias: The positions arrays are one element longer than
@@ -732,243 +736,262 @@ int xbaeCalculateLabelMaxLines(String *labels, XmString *xmlabels, int n_labels)
         return max_lines;
 }
 
-/*
- * Convert the coordinates in an event to coordinates relative to the matrix window
- */
-
-/* ARGSUSED */
-Boolean xbaeEventToMatrixXY(XbaeMatrixWidget mw, XEvent * event, int *x, int *y)
-{
-        switch (event->type) {
-        case ButtonPress:
-        case ButtonRelease:
-                *x = event->xbutton.x;
-                *y = event->xbutton.y;
-                break;
-        case KeyPress:
-        case KeyRelease:
-                *x = event->xkey.x;
-                *y = event->xkey.y;
-                break;
-        case MotionNotify:
-                *x = event->xmotion.x;
-                *y = event->xmotion.y;
-                break;
-        default:
-                return False;
-        }
-
-        if (event->xbutton.window == XtWindow(TextChild(mw))) {
-                *x += xbaeColumnToMatrixX(mw, mw->matrix.current_column);
-                *y += xbaeRowToMatrixY(mw, mw->matrix.current_row);
-        } else if (event->xbutton.window == XtWindow(LeftClip(mw))) {
-                *x += FIXED_COLUMN_POSITION(mw);
-                *y += NON_FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(RightClip(mw))) {
-                *x += TRAILING_FIXED_COLUMN_POSITION(mw);
-                *y += NON_FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(TopClip(mw))) {
-                *x += NON_FIXED_COLUMN_POSITION(mw);
-                *y += FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(BottomClip(mw))) {
-                *x += NON_FIXED_COLUMN_POSITION(mw);
-                *y += TRAILING_FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(ClipChild(mw))) {
-                *x += NON_FIXED_COLUMN_POSITION(mw);
-                *y += NON_FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(RowLabelClip(mw))) {
-                *x += VERT_SB_OFFSET(mw);
-                *y += NON_FIXED_ROW_POSITION(mw);
-        } else if (event->xbutton.window == XtWindow(ColumnLabelClip(mw))) {
-                *x += NON_FIXED_COLUMN_POSITION(mw);
-                *y += HORIZ_SB_OFFSET(mw);
-        } else if (event->xbutton.window != XtWindow(mw)) {
-                return False;
-        }
-
-        return True;
-}
-
-/* ARGSUSED */
-Boolean xbaeNewEventToMatrixXY(XbaeMatrixWidget mw, Widget w, XEvent * event, int *x, int *y)
-{
-        switch (event->type) {
-        case ButtonPress:
-        case ButtonRelease:
-                *x = event->xbutton.x;
-                *y = event->xbutton.y;
-                break;
-        case KeyPress:
-        case KeyRelease:
-                *x = event->xkey.x;
-                *y = event->xkey.y;
-                break;
-        case MotionNotify:
-                *x = event->xmotion.x;
-                *y = event->xmotion.y;
-                break;
-        default:
-                return False;
-        }
-
-        for(; w && w != (Widget) mw; w = XtParent(w)) {
-                *x += w->core.x;
-                *y += w->core.y;
-        }
-        
-        return w == (Widget) mw;
-}
-
 /*
  * Convert a matrix window x position to a column.
- * Return true if x falls in a column or a row label. If so adjust x so that it's
- * relative to the column/label. If not return false and leave x alone
+ * Return the clip region if x falls in a column or a row label. If so adjust x so that it's
+ * relative to the column/label. If not return 0 and leave x alone
  */
-
-/* ARGSUSED */
-Boolean xbaeMatrixXtoColumn(XbaeMatrixWidget mw, int *x, int *column)
+int xbaeMatrixXtoColumn(XbaeMatrixWidget mw, int *x, int *column)
 {
-
         if (*x >= VERT_SB_OFFSET(mw) && *x < VERT_SB_OFFSET(mw) + ROW_LABEL_WIDTH(mw)) {
-                /* It's in the row labels */
+                /* In the row labels */
                 *column = -1;
-
-                /* Make x relative to the row labels */
                 *x -= VERT_SB_OFFSET(mw);
-        } else {
-                /* Transform x so it's relative to the virtual matrix */
-                if (*x >= FIXED_COLUMN_POSITION(mw)
-                    && *x < FIXED_COLUMN_POSITION(mw) + VISIBLE_FIXED_COLUMN_WIDTH(mw)) {
-                        *x -= FIXED_COLUMN_POSITION(mw);
-                        /* Check if we are not in the horizontal fill */
-                        if(*x >= COLUMN_POSITION(mw, mw->matrix.fixed_columns)) {
-                                *column = mw->matrix.fixed_columns - 1;
-                                *x -= COLUMN_POSITION(mw, *column);
-                                return True;
-                        }
-                } else if (*x >= TRAILING_FIXED_COLUMN_POSITION(mw)
-                           && *x < TRAILING_FIXED_COLUMN_POSITION(mw)
-                                   + VISIBLE_TRAILING_FIXED_COLUMN_WIDTH(mw)) {
-                        *x -= TRAILING_FIXED_COLUMN_POSITION(mw) 
-                              - COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw));
-                        /* Check if we are not in the horizontal fill */
-                        if(*x >= COLUMN_POSITION(mw, mw->matrix.columns)) {
-                                *column = mw->matrix.columns - 1;
-                                *x -= COLUMN_POSITION(mw, *column);
-                                return True;
-                        }
-                } else if (*x >= NON_FIXED_COLUMN_POSITION(mw)
-                           && *x < NON_FIXED_COLUMN_POSITION(mw) + VISIBLE_NON_FIXED_WIDTH(mw)) {
-                        *x -= NON_FIXED_COLUMN_POSITION(mw) - HORIZ_ORIGIN(mw)
-                             - FIXED_COLUMN_WIDTH(mw);
-                        /* Check if we are not in the horizontal fill */
-                        if(*x >= COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw))) {
-                                *column = TRAILING_COLUMN_ORIGIN(mw) - 1;
-                                *x -= COLUMN_POSITION(mw, *column);
-                                return True;
-                        }
+                return CLIP_ROW_LABELS;
+        } else if (*x >= FIXED_COLUMN_POSITION(mw)
+                   && *x < FIXED_COLUMN_POSITION(mw) + VISIBLE_FIXED_COLUMN_WIDTH(mw)) {
+                /* In the fixed columns */
+                *x -= FIXED_COLUMN_POSITION(mw);
+                if(*x >= COLUMN_POSITION(mw, mw->matrix.fixed_columns)) {
+                        /* We are in the horizontal fill */
+                        *column = mw->matrix.fixed_columns - 1;
                 } else {
-                        *column = -2;
-                        return False;
+                        /* Get the column it corresponds to */
+                        *column = xbaeXtoCol(mw, *x);
                 }
-                /* Get the column it corresponds to */
-                *column = xbaeXtoCol(mw, *x);
-
-                /* Make x relative to the that column */
                 *x -= COLUMN_POSITION(mw, *column);
+                return CLIP_FIXED_COLUMNS;
+        } else if (*x >= TRAILING_FIXED_COLUMN_POSITION(mw)
+                   && *x < TRAILING_FIXED_COLUMN_POSITION(mw) + VISIBLE_TRAILING_FIXED_COLUMN_WIDTH(mw)) {
+                /* In the trailing fixed columns */
+                *x -= TRAILING_FIXED_COLUMN_POSITION(mw) - COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw));
+                if(*x >= COLUMN_POSITION(mw, mw->matrix.columns)) {
+                        /* We are in the horizontal fill */
+                        *column = mw->matrix.columns - 1;
+                } else {
+                        /* Get the column it corresponds to */
+                        *column = xbaeXtoCol(mw, *x);
+                }
+                *x -= COLUMN_POSITION(mw, *column);
+                return CLIP_TRAILING_FIXED_COLUMNS;
+        } else if (*x >= NON_FIXED_COLUMN_POSITION(mw)
+                   && *x < NON_FIXED_COLUMN_POSITION(mw) + VISIBLE_NON_FIXED_WIDTH(mw)) {
+                /* In the non fixed columns */
+                *x -= NON_FIXED_COLUMN_POSITION(mw) - HORIZ_ORIGIN(mw) - FIXED_COLUMN_WIDTH(mw);
+                if(*x >= COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw))) {
+                        /* We are in the horizontal fill */
+                        *column = TRAILING_COLUMN_ORIGIN(mw) - 1;
+                } else {
+                        /* Get the column it corresponds to */
+                        *column = xbaeXtoCol(mw, *x);
+                }
+                *x -= COLUMN_POSITION(mw, *column);
+                return CLIP_VISIBLE_WIDTH;
         }
 
-        return True;
+        *column = -2;
+        return 0;
 }
 
 /*
  * Convert a matrix window y position to a row.
- * Return true if y falls in a row or a column label. If so adjust y so that it's 
- * relative to the row/label. If not return false and leave y alone
+ * Return the clip region if y falls in a row or a column label. If so adjust y so that it's 
+ * relative to the row/label. If not return 0 and leave y alone
  */
-
-/* ARGSUSED */
-Boolean xbaeMatrixYtoRow(XbaeMatrixWidget mw, int *y, int *row)
+int xbaeMatrixYtoRow(XbaeMatrixWidget mw, int *y, int *row)
 {
-
         if (*y >= HORIZ_SB_OFFSET(mw) && *y < HORIZ_SB_OFFSET(mw) + COLUMN_LABEL_HEIGHT(mw)) {
-                /* It's in the column labels */
+                /* In the column labels */
                 *row = -1;
-
-                /* Make y relative to the column labels */
                 *y -= HORIZ_SB_OFFSET(mw);
-        } else {
-                /* Transform the y coordinate so it's relative to the virtual matrix */
-                if (*y >= FIXED_ROW_POSITION(mw) && *y < FIXED_ROW_POSITION(mw) + VISIBLE_FIXED_ROW_HEIGHT(mw)) {
-                        *y -= FIXED_ROW_POSITION(mw);
-                        /* Check if we are not in the vertical fill */
-                        if (*y >= ROW_POSITION(mw, mw->matrix.fixed_rows)) {
-                                *row = mw->matrix.fixed_rows - 1;
-                                *y -= ROW_POSITION(mw, *row);
-                                return True;
-                        }
-                } else if (*y >= TRAILING_FIXED_ROW_POSITION(mw)
-                           && *y < TRAILING_FIXED_ROW_POSITION(mw) 
-                                   + VISIBLE_TRAILING_FIXED_ROW_HEIGHT(mw)) {
-                        *y -= TRAILING_FIXED_ROW_POSITION(mw) 
-                              - ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw));
-                        /* Check if we are not in the vertical fill */
-                        if (*y >= ROW_POSITION(mw, mw->matrix.rows)) {
-                                *row = mw->matrix.rows - 1;
-                                *y -= ROW_POSITION(mw, *row);
-                                return True;
-                        }
-                 } else if (*y >= NON_FIXED_ROW_POSITION(mw)
-                           && *y < NON_FIXED_ROW_POSITION(mw) + VISIBLE_NON_FIXED_HEIGHT(mw)) {
-                        *y -= NON_FIXED_ROW_POSITION(mw) - VERT_ORIGIN(mw) - FIXED_ROW_HEIGHT(mw);
-                        /* Check if we are not in the vertical fill */
-                        if (*y >= ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw))) {
-                                *row = TRAILING_ROW_ORIGIN(mw) - 1;
-                                *y -= ROW_POSITION(mw, *row);
-                                return True;
-                        }
+                return CLIP_COLUMN_LABELS;
+        } else if (*y >= FIXED_ROW_POSITION(mw) && *y < FIXED_ROW_POSITION(mw) + VISIBLE_FIXED_ROW_HEIGHT(mw)) {
+                /* In the fixed rows */
+                *y -= FIXED_ROW_POSITION(mw);
+                if (*y >= ROW_POSITION(mw, mw->matrix.fixed_rows)) {
+                        /* We are in the vertical fill */
+                        *row = mw->matrix.fixed_rows - 1;
                 } else {
-                        *row = -2;
-                        return False;
+                        /* Get the row it corresponds to */
+                        *row = xbaeYtoRow(mw, *y);
                 }
-                /* Get the row it corresponds to */
-                *row = xbaeYtoRow(mw, *y);
-
-                /* Make y relative to the that row */
                 *y -= ROW_POSITION(mw, *row);
+                return CLIP_FIXED_ROWS;
+        } else if (*y >= TRAILING_FIXED_ROW_POSITION(mw)
+                   && *y < TRAILING_FIXED_ROW_POSITION(mw) + VISIBLE_TRAILING_FIXED_ROW_HEIGHT(mw)) {
+                /* In the trailing fixed rows */
+                *y -= TRAILING_FIXED_ROW_POSITION(mw) - ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw));
+                if (*y >= ROW_POSITION(mw, mw->matrix.rows)) {
+                        /* We are in the vertical fill */
+                        *row = mw->matrix.rows - 1;
+                } else {
+                        /* Get the row it corresponds to */
+                        *row = xbaeYtoRow(mw, *y);
+                }
+                *y -= ROW_POSITION(mw, *row);
+                return CLIP_TRAILING_FIXED_ROWS;
+         } else if (*y >= NON_FIXED_ROW_POSITION(mw)
+                   && *y < NON_FIXED_ROW_POSITION(mw) + VISIBLE_NON_FIXED_HEIGHT(mw)) {
+                /* In the non fixed rows */
+                *y -= NON_FIXED_ROW_POSITION(mw) - VERT_ORIGIN(mw) - FIXED_ROW_HEIGHT(mw);
+                if (*y >= ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw))) {
+                        /* We are in the vertical fill */
+                        *row = TRAILING_ROW_ORIGIN(mw) - 1;
+                } else {
+                        /* Get the row it corresponds to */
+                        *row = xbaeYtoRow(mw, *y);
+                }
+                *y -= ROW_POSITION(mw, *row);
+                return CLIP_VISIBLE_HEIGHT;
         }
 
-        return True;
+        *row = -2;
+        return 0;
 }
 
 /*
  * Convert a matrix window x, y to a row, column pair.
- * Return true if x, y falls in a cell. If so adjust x and y so they are
- * relative to the cell.
- * Return false if x, y falls in a label. Then one of row/column is -1. If so
- * adjust x and y so they are relative to the labels origin.
- * Return false if x, y doesn't fall in a cell nor a label and set both row and 
- * col to -1 leaving x and y alone.
+ * Return the clip region if x, y falls in a cell or a label. If so adjust x 
+ * and y so they are relative to the cell/label. If not return 0 and leave
+ * x and y alone.
  */
 
 /* ARGSUSED */
-Boolean xbaeMatrixXYToRowCol(XbaeMatrixWidget mw, int *x, int *y, int *row, int *column)
+int xbaeMatrixXYToRowCol(XbaeMatrixWidget mw, int *x, int *y, int *row, int *column)
 {
         int ret_x = *x;
         int ret_y = *y;
 
-        if (!xbaeMatrixXtoColumn(mw, &ret_x, column) || !xbaeMatrixYtoRow(mw, &ret_y, row)
-            || (*row == -1 && *column == -1)) {
+        int row_region = xbaeMatrixYtoRow(mw, &ret_y, row);
+        int column_region = xbaeMatrixXtoColumn(mw, &ret_x, column);
+
+        if (row_region == 0 || column_region == 0 
+            || (row_region == CLIP_COLUMN_LABELS && column_region == CLIP_ROW_LABELS)) {
                 /* Not a cell nor a label */
                 *row = -1;
                 *column = -1;
-                return False;
+                return 0;
         }
 
         *x = ret_x;
         *y = ret_y;
 
-        return *row != -1 && *column != -1;
+        return row_region | column_region;
+}
+
+int xbaeEventToRowColumn(Widget w, XEvent * event, int *row, int *column, int *x, int *y)
+{
+        XbaeMatrixWidget mw;
+
+        switch (event->type) {
+        case KeyPress:
+        case KeyRelease:
+                *x = 0;
+                *y = 0;
+                break;
+        case ButtonPress:
+        case ButtonRelease:
+                *x = event->xbutton.x;
+                *y = event->xbutton.y;
+                break;
+        case MotionNotify:
+                *x = event->xmotion.x;
+                *y = event->xmotion.y;
+                break;
+        default:
+                return 0;
+        }
+
+        if (XtIsSubclass(w, xbaeMatrixWidgetClass)) {
+                /* The event occured in the matrix widget */
+                
+                mw = (XbaeMatrixWidget) w;
+
+                return xbaeMatrixXYToRowCol(mw, x, y, row, column);
+        } else {
+                /* 
+                 * The event didn't occur in a matrix widget. Find an ancestor that is the child
+                 + of a matrix widget
+                 */
+
+                while (XtParent(w) != NULL && !XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass)) {
+                        *x += w->core.x;
+                        *y += w->core.y;
+                        w = XtParent(w);
+                }
+
+                mw = (XbaeMatrixWidget) XtParent(w);
+
+                if (mw != NULL) {
+                        XtVaGetValues(w, XmNattachRow, row, XmNattachColumn, column, NULL);
+                        
+                        return (*row == -1 || *column ==-1) ? 0 : xbaeCellClip(mw, *row, *column);
+                }
+        }
+        
+        return 0;
+}
+
+/*
+ * Convert the coordinates in an event to coordinates relative to the matrix window
+ */
+XbaeMatrixWidget xbaeEventToMatrixXY(Widget w, XEvent * event, int *x, int *y)
+{
+        XbaeMatrixWidget mw;
+        *x = 0;
+        *y = 0;
+
+        if (XtIsSubclass(w, xbaeMatrixWidgetClass)) {
+                mw = (XbaeMatrixWidget) w;
+        } else if (XtParent(w)
+                   && XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass)
+                   && w == TextField((XbaeMatrixWidget) XtParent(w))) {
+                int current_row, current_column;                
+                mw = (XbaeMatrixWidget) XtParent(w);
+                
+                XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
+                
+                *x = xbaeColumnToMatrixX(mw, current_column);
+                *y = xbaeRowToMatrixY(mw, current_row);
+        } else {
+                do {
+                        *x += w->core.x;
+                        *y += w->core.y;
+                        w = XtParent(w);
+                } while (w && !XtIsSubclass(w, xbaeMatrixWidgetClass));
+
+                mw = (XbaeMatrixWidget) w;
+        }
+
+        switch (event->type) {
+        case KeyPress:
+        case KeyRelease:
+                break;
+        case ButtonPress:
+        case ButtonRelease:
+                *x += event->xbutton.x;
+                *y += event->xbutton.y;
+                break;
+        case MotionNotify:
+                *x += event->xmotion.x;
+                *y += event->xmotion.y;
+                break;
+        default:
+                return NULL;
+        }
+
+        return mw;
+}
+
+/*
+ * Find the matrix widget by walking the Widget tree 
+ */
+XbaeMatrixWidget xbaeEventToMatrixWidget(Widget w, XEvent *event) {
+        while (w != NULL && !XtIsSubclass(w, xbaeMatrixWidgetClass)) {
+            w = XtParent(w);
+        }
+        
+        return (XbaeMatrixWidget) w;
 }
 
 /*
@@ -1009,16 +1032,10 @@ int xbaeRowToMatrixY(XbaeMatrixWidget mw, int row)
         return y;
 }
 
-void xbaeRowColToMatrixXY(XbaeMatrixWidget mw, int row, int column, int *widget_x, int *widget_y)
-{
-        *widget_x = xbaeColumnToMatrixX(mw, column);
-        *widget_y = xbaeRowToMatrixY(mw, row);
-}
-
 /*
  * Convert a row/column cell position to the x/y of its upper left corner
  * wrt the window it will be drawn in (either the matrix window for
- * fixed cells, or the clip window for non-fixed).
+ * totally fixed cells/labels, or a clip window for non-fixed).
  */
 Widget xbaeRowColToClipXY(XbaeMatrixWidget mw, int row, int column, int *x, int *y)
 {
@@ -1052,11 +1069,21 @@ void xbaePositionWidgetOverCell(XbaeMatrixWidget mw, Widget w, int row, int colu
                 height += EMPTY_HEIGHT(mw);
         }
         
+        #if 1
+        /* This works, but the TextField flickers when scrolling (offset by labelsize + fixed) */
         XtConfigureWidget(w,
            /* position */ x, y,
            /* size     */ width, height,
            /* bw       */ XtBorderWidth(w));
-
+        #else 
+        /* The TextField doesn't flicker but toggle button cell widgets don't work anymore */
+        XtResizeWidget(w,
+            /* size     */ width, height, 
+            /* bw       */ XtBorderWidth(w));
+        #endif
+        
+        XtVaSetValues(w, XmNattachRow, row, XmNattachColumn, column, NULL);
+        
         if (XtWindow(new_parent)) {
                 if (new_parent != (Widget) mw) {
                         /* The widget is drawn in one of the clip windows */
@@ -1076,16 +1103,17 @@ void xbaePositionCellWidget(XbaeMatrixWidget mw, int row, int column)
         }
 }
 
-void xbaePositionTextChild(XbaeMatrixWidget mw)
+void xbaePositionTextField(XbaeMatrixWidget mw)
 {
-        xbaePositionWidgetOverCell(mw, TextChild(mw),
-                                   mw->matrix.current_row, 
-                                   mw->matrix.current_column);
+        int current_row, current_column;
+        XtVaGetValues(TextField(mw), XmNattachRow, &current_row, XmNattachColumn, &current_column, NULL);
+
+        xbaePositionWidgetOverCell(mw, TextField(mw), current_row, current_column);
 
         /*
-         * The textChild is now visible
+         * The TextField is now visible
          */
-        mw->matrix.text_child_is_mapped = True;
+        mw->matrix.text_field_is_mapped = True;
 }
 
 void xbaeHideCellWidget(XbaeMatrixWidget mw, Widget w)
@@ -1094,32 +1122,34 @@ void xbaeHideCellWidget(XbaeMatrixWidget mw, Widget w)
         Dimension height = XtHeight(w);
         Dimension bw = XtBorderWidth(w);
 
+        XtVaSetValues(w, XmNattachRow, -1, XmNattachColumn, -1, NULL);
+
         XtConfigureWidget(w,
                          /* position */	-1 - width - bw, -1 - height - bw,
                          /* size */	width, height,
                          /* bw */	bw);
 }
 
-void xbaeHideTextChild(XbaeMatrixWidget mw)
+void xbaeHideTextField(XbaeMatrixWidget mw)
 {
         /*
-         * Let Xt believe the textChild is still visible so we can traverse to it
+         * Let Xt believe the TextField is still visible so we can traverse to it
          */
-        XtConfigureWidget(TextChild(mw),
+        XtConfigureWidget(TextField(mw),
            /* position */ 0, 0,
            /* size     */ 1, 1,
-           /* bw       */ XtBorderWidth(TextChild(mw)));
+           /* bw       */ XtBorderWidth(TextField(mw)));
         
         /*
          * But have X not show the window
          */
-        if (XtIsRealized(TextChild(mw)))
-		XReparentWindow(XtDisplay(mw), XtWindow(TextChild(mw)), XtWindow(mw), -1, -1);
+        if (XtIsRealized(TextField(mw)))
+                XReparentWindow(XtDisplay(mw), XtWindow(TextField(mw)), XtWindow(mw), -1, -1);
         
         /*
-         * The textChild is now hidden
+         * The TextField is now hidden
          */
-        mw->matrix.text_child_is_mapped = False;
+        mw->matrix.text_field_is_mapped = False;
 }
 
 void xbaeSetInitialFocus(XbaeMatrixWidget mw)
@@ -1131,7 +1161,7 @@ void xbaeSetInitialFocus(XbaeMatrixWidget mw)
         if (mw->matrix.per_cell && mw->matrix.per_cell[row][column].widget) {
                 widget = mw->matrix.per_cell[row][column].widget;
         } else {
-                widget = TextChild(mw);
+                widget = TextField(mw);
         }
 
         if (widget != mw->manager.initial_focus) {
@@ -1218,68 +1248,15 @@ void xbaeObjectUnlock(Widget w)
 #endif
 }
 
-/* Some string functions, since strcasecmp()/strncasecmp() are
-   not always available */
-int _xbaeStrcasecmp(const char *s1, const char *s2)
-{
-#ifdef HAVE_STRNCASECMP
-        return strcasecmp(s1, s2);
-#else
-#ifdef HAVE_STRNICMP
-        return stricmp(s1, s2);
-#else
-        int c1, c2;
-
-        while (*s1 && *s2) {
-                c1 = tolower(*s1);
-                c2 = tolower(*s2);
-                if (c1 != c2) {
-                        return (c1 - c2);
-                }
-                s1++;
-                s2++;
-        }
-        return (int) (*s1 - *s2);
-#endif
-#endif
-}
-
-
-int _xbaeStrncasecmp(const char *s1, const char *s2, size_t count)
-{
-#ifdef HAVE_STRNCASECMP
-        return strncasecmp(s1, s2, count);
-#else
-#ifdef HAVE_STRNICMP
-        return strnicmp(s1, s2, count);
-#else
-        int c1, c2;
-        int i = 0;
-
-        while (*s1 && *s2 && i < (int) count) {
-                c1 = tolower(*s1);
-                c2 = tolower(*s2);
-                if (c1 != c2) {
-                        return (c1 - c2);
-                }
-                s1++;
-                s2++;
-                i++;
-        }
-        return (int) (*s1 - *s2);
-#endif
-#endif
-}
-
-void xbaeScrollRows(XbaeMatrixWidget mw, Boolean Up, int step)
+void xbaeScrollRows(XbaeMatrixWidget mw, int step)
 {
         int value, slider_size, increment, page_increment, limit;
-        XtVaGetValues(VertScrollChild(mw), (Up ? XmNminimum : XmNmaximum), &limit, NULL);
+        XtVaGetValues(VertScrollChild(mw), (step < 0 ? XmNminimum : XmNmaximum), &limit, NULL);
         XmScrollBarGetValues(VertScrollChild(mw), &value, &slider_size, &increment,
                              &page_increment);
-        if (Up) {
+        if (step < 0) {
                 XmScrollBarSetValues(VertScrollChild(mw),
-                                     ((value - step < limit) ? limit : value - step), slider_size,
+                                     ((value + step < limit) ? limit : value + step), slider_size,
                                      increment, page_increment, True);
         } else {
                 limit -= slider_size;
@@ -1289,15 +1266,15 @@ void xbaeScrollRows(XbaeMatrixWidget mw, Boolean Up, int step)
         }
 }
 
-void xbaeScrollColumns(XbaeMatrixWidget mw, Boolean Left, int step)
+void xbaeScrollColumns(XbaeMatrixWidget mw, int step)
 {
         int value, slider_size, increment, page_increment, limit;
-        XtVaGetValues(HorizScrollChild(mw), (Left ? XmNminimum : XmNmaximum), &limit, NULL);
+        XtVaGetValues(HorizScrollChild(mw), (step < 0 ? XmNminimum : XmNmaximum), &limit, NULL);
         XmScrollBarGetValues(HorizScrollChild(mw), &value, &slider_size, &increment,
                              &page_increment);
-        if (Left) {
+        if (step < 0) {
                 XmScrollBarSetValues(HorizScrollChild(mw),
-                                     ((value - step < limit) ? limit : value - step), slider_size,
+                                     ((value + step < limit) ? limit : value + step), slider_size,
                                      increment, page_increment, True);
         } else {
                 limit -= slider_size;
